@@ -25,6 +25,8 @@
 #include "Toolkit.h"
 #include "RasSrv.h"
 #include "Routing.h"
+#include "gkacct.h"
+#include "capctrl.h"
 #include "rwlock.h"
 #include "gk.h"
 #include "h323util.h"
@@ -140,7 +142,7 @@ public:
 	*/
 	void OnCommand(
 		/// command to be executed
-		const PString& cmd
+		PString cmd
 		);
 
 	/** @return
@@ -734,6 +736,10 @@ void GkStatus::OnStart()
 	m_commands["printpc"] = e_PrintPrefixCapacities;
 	m_commands["printcc"] = e_PrintCapacityControlRules;
 	m_commands["sendproceeding"] = e_SendProceeding;
+	m_commands["getauthinfo"] = e_GetAuthInfo;
+	m_commands["gai"] = e_GetAuthInfo;
+	m_commands["getacctinfo"] = e_GetAcctInfo;
+	m_commands["gci"] = e_GetAcctInfo;
 }
 
 void GkStatus::ReadSocket(
@@ -831,6 +837,11 @@ bool StatusClient::ReadCommand(
 					}
 				}
 				break;
+			case '\x04':	// Ctrl-D
+				cmd = "exit";
+				m_lastCmd = cmd;
+				m_currentCmd = PString();
+				return true;
 			default:
 				byte = char(ch);
 				m_currentCmd += byte;
@@ -924,7 +935,7 @@ bool StatusClient::Authenticate()
 
 void StatusClient::OnCommand(
 	/// command to be executed
-	const PString& cmd
+	PString cmd
 	)
 {
 	{
@@ -933,6 +944,10 @@ void StatusClient::OnCommand(
 	}
 	// problem - if the ExecCommand does not get executed for some reason,
 	// m_numExecutingCommands will not decrement
+	
+	// make sure this PString doesn't share memory with other PStrings
+	// when we send it into another thread
+	cmd.MakeUnique();
 	CreateJob(this, &StatusClient::ExecCommand, cmd, "StatusCmd " + cmd);
 }
 
@@ -1257,12 +1272,43 @@ void StatusClient::ExecCommand(
 		break;
 	case GkStatus::e_Reload:
 		{
-			ConfigReloadMutex.EndRead();
-			ReloadHandler();
-			ConfigReloadMutex.StartRead();
+			if (args.GetSize() >= 2) {
+				ConfigReloadMutex.StartWrite();
+				Toolkit::Instance()->PrepareReloadConfig();
+				if (args[1] == "acctconfig") {
+					GkAcctLoggerList *acctList = RasServer::Instance()->GetAcctList();
+					if (acctList)
+						acctList->OnReload();
+					PTRACE(1, "STATUS\tAcct Config reloaded.");
+					m_gkStatus->SignalStatus("Acct Config reloaded.\r\n");
+				} else if (args[1] == "authconfig") {
+					GkAuthenticatorList *authList = RasServer::Instance()->GetAuthList();
+					if (authList)
+						authList->OnReload();
+					PTRACE(1, "STATUS\tAuth Config reloaded.");
+					m_gkStatus->SignalStatus("Auth Config reloaded.\r\n");
+				} else if (args[1] == "capconfig") {
+					CapacityControl::Instance()->LoadConfig();
+					PTRACE(1, "STATUS\tCapacityControl Config reloaded.");
+					m_gkStatus->SignalStatus("CapacityControl Config reloaded.\r\n");
+				} else if (args[1] == "epconfig") {
+					CallTable::Instance()->LoadConfig();
+					RegistrationTable::Instance()->LoadConfig();
+					CallTable::Instance()->UpdatePrefixCapacityCounters();
+					PTRACE(1, "STATUS\tEP Config reloaded.");
+					m_gkStatus->SignalStatus("EP Config reloaded.\r\n");
+				} else {
+					CommandError("Syntax Error: Reload <AcctConfig|AuthConfig|CapConfig|EpConfig>");
+				}
+				ConfigReloadMutex.EndWrite();
+			} else {
+				ConfigReloadMutex.EndRead();	// ReloadHandler() re-aquires a write lock
+				ReloadHandler();
+				ConfigReloadMutex.StartRead();
+				PTRACE(1, "STATUS\tFull Config reloaded.");
+				m_gkStatus->SignalStatus("Full Config reloaded.\r\n");
+			}
 		}
-		PTRACE(1, "STATUS\tConfig reloaded.");
-		m_gkStatus->SignalStatus("Config reloaded.\r\n");
 		break;
 	case GkStatus::e_Shutdown:
 		if (!Toolkit::AsBool(GkConfig()->GetString(authsec, "Shutdown", "1"))) {
@@ -1408,6 +1454,18 @@ void StatusClient::ExecCommand(
 	case GkStatus::e_PrintCapacityControlRules:
 		SoftPBX::PrintCapacityControlRules(this);
 		break;
+	case GkStatus::e_GetAuthInfo:
+		if (args.GetSize() == 2)
+			WriteString(RasServer::Instance()->GetAuthInfo(args[1]));
+		else
+			CommandError("Syntax Error: GetAuthInfo|gai AUTH_MODULE_NAME");
+		break;
+	case GkStatus::e_GetAcctInfo:
+		if (args.GetSize() == 2)
+			WriteString(RasServer::Instance()->GetAcctInfo(args[1]));
+		else
+			CommandError("Syntax Error: GetAcctInfo|gci ACCT_MODULE_NAME");
+		break;
 	default:
 		// commmand not recognized
 		CommandError("Error: Unknown command '" + cmd + "'");
