@@ -45,21 +45,27 @@ MakeCallEndPoint::MakeCallEndPoint() : Singleton<MakeCallEndPoint>("MakeCallEndP
 	}
 
 	PString gkName = GkConfig()->GetString("CTI::MakeCall", "Gatekeeper", "127.0.0.1");
-	if (SetGatekeeper(gkName, rasChannel))
+	if (SetGatekeeper(gkName, rasChannel)) {
 		PTRACE(3, "MakeCallEndpoint: Gatekeeper set: " << *gatekeeper);
-	else {
+		isRegistered = TRUE;
+	} else {
 		PTRACE(1, "MakeCallEndpoint: Error registering with gatekeeper at \"" << gkName << '"');
+		isRegistered = FALSE;
 	}
 }
 
 void MakeCallEndPoint::ThirdPartyMakeCall(PString & user1, PString & user2)
 {
+	if (!IsRegisteredWithGk()) {
+		PTRACE(1, "MakeCallEndpoint: Can't MakeCall when not registered with gatekeeper");
+		return;
+	}
 	PString newToken;
 	MakeCall(user1, newToken);
 	AddDestination(newToken, user2);
 }
 
-BOOL MakeCallEndPoint::GatekeeperIsRegistered(void)
+PBoolean MakeCallEndPoint::IsRegisteredWithGk(void) const
 {
 	return isRegistered;
 }	
@@ -87,7 +93,7 @@ PString MakeCallEndPoint::GetDestination(PString token)
 }
 
 
-BOOL MakeCallEndPoint::OnIncomingCall(H323Connection & connection,
+PBoolean MakeCallEndPoint::OnIncomingCall(H323Connection & connection,
                                         const H323SignalPDU &,
                                         H323SignalPDU &)
 {
@@ -96,7 +102,7 @@ BOOL MakeCallEndPoint::OnIncomingCall(H323Connection & connection,
 }
 
 
-BOOL MakeCallEndPoint::OnConnectionForwarded(H323Connection & connection,
+PBoolean MakeCallEndPoint::OnConnectionForwarded(H323Connection & connection,
                                                const PString & forwardParty,
                                                const H323SignalPDU & /*pdu*/)
 {
@@ -139,8 +145,8 @@ void MakeCallEndPoint::OnRegistrationReject()
 	isRegistered = FALSE;
 }
 
-BOOL MakeCallEndPoint::OpenAudioChannel(H323Connection & connection,
-                                          BOOL isEncoding,
+PBoolean MakeCallEndPoint::OpenAudioChannel(H323Connection & connection,
+                                          PBoolean isEncoding,
                                           unsigned bufferSize,
                                           H323AudioCodec & codec)
 {
