@@ -37,7 +37,7 @@
 #include "cisco.h"
 #include "h323util.h"
 #include "Neighbor.h"
-#include "pwlib_compat.h"
+#include "config.h"
 
 #ifdef hasH460
   #include <h460/h4601.h>
@@ -489,6 +489,12 @@ bool GnuGK::OnSendingLRQ(H225_LocationRequest & lrq, const AdmissionRequest & re
        lrq.IncludeOptionalField(H225_LocationRequest::e_genericData); 
        H225_ArrayOf_GenericData & data = lrq.m_genericData;
        PINDEX lastPos = 0;
+       
+	   /// STD24  'NAT Support
+	   H460_FeatureStd std24 = H460_FeatureStd(24);
+	   lastPos++;
+	   data.SetSize(lastPos);
+	   data[lastPos-1] = std24;
 
 	   /// OID9  'Remote application info
 	   H460_FeatureOID foid9 = H460_FeatureOID(OID9);
@@ -1099,7 +1105,7 @@ public:
 
 private:
 	// override from class Policy
-	virtual bool IsActive();
+	virtual bool IsActive() const;
 
 	virtual bool OnRequest(AdmissionRequest &);
 	virtual bool OnRequest(LocationRequest &);
@@ -1117,7 +1123,7 @@ NeighborPolicy::NeighborPolicy() : m_neighbors(*RasServer::Instance()->GetNeighb
 	m_name = "Neighbor";
 }
 
-bool NeighborPolicy::IsActive()
+bool NeighborPolicy::IsActive() const
 {
 	return !m_neighbors.empty();
 }
@@ -1173,11 +1179,13 @@ bool NeighborPolicy::OnRequest(LocationRequest & lrq_obj)
 	List::iterator iter = find_if(m_neighbors.begin(), m_neighbors.end(), bind2nd(mem_fun(&Neighbor::IsAcceptable), ras));
 	Neighbor *requester = (iter != m_neighbors.end()) ? *iter : 0;
 	int hopCount = 0;
-	if (requester)
-		if (requester->ForwardLRQ() < 0)
+	if (requester) {
+		if (requester->ForwardLRQ() < 0) {
 			return false;
-		else if (requester->ForwardLRQ() > 0)
+		} else if (requester->ForwardLRQ() > 0) {
 			hopCount = 1;
+		}
+	}
 
 	H225_LocationRequest & lrq = (*ras)->m_recvRAS;
 	if (lrq.HasOptionalField(H225_LocationRequest::e_hopCount)) {
@@ -1364,7 +1372,7 @@ bool SRVPolicy::FindByAliases(
 				             PString called = ls[i].Left(at);
 						     PINDEX k = 0;
 						     for (k = 0; k < called.GetLength(); ++k)
-								if (!isdigit(alias[k]))
+								if (!isdigit(static_cast<unsigned char>(alias[k])))
 									break;
 								if (k >= alias.GetLength())
 									H323SetAliasAddress(called, aliases[i]);
@@ -1393,7 +1401,7 @@ bool SRVPolicy::FindByAliases(
 				     PString called = cs[j].Left(in);
 					  PINDEX l = 0;
 						for (l = 0; l < called.GetLength(); ++l)
-							if (!isdigit(alias[l]))
+							if (!isdigit(static_cast<unsigned char>(alias[l])))
 								break;
 						if (l >= alias.GetLength())
 						  H323SetAliasAddress(called, aliases[i]);
