@@ -40,7 +40,7 @@
 #include "gkacct.h"
 #include "gktimer.h"
 #include "RasSrv.h"
-#include "pwlib_compat.h"
+#include "config.h"
 
 #ifdef hasH460
   #include <h460/h4601.h>
@@ -377,11 +377,6 @@ bool RasMsg::IsFrom(const PIPSocket::Address & addr, WORD pt) const
 	return (addr == m_msg->m_peerAddr) && (pt == 0 || pt == m_msg->m_peerPort);
 }
 
-void RasMsg::GetRecvAddress(PIPSocket::Address & addr, WORD & pt) const
-{
-	addr = m_msg->m_peerAddr, pt = m_msg->m_peerPort;
-}
-
 void RasMsg::GetRasAddress(H225_TransportAddress & result) const
 {
 	result = SocketToH225TransportAddr(m_msg->m_localAddr, m_msg->m_socket->GetPort());
@@ -508,13 +503,16 @@ bool GkInterface::CreateListeners(RasServer *RasSrv)
 
 	// MulticastListener::GetPort() didn't return the real multicast port
 	m_multicastPort = multicastPort;
-	if (m_rasListener && m_callSignalListener)
+	if (m_rasListener && m_callSignalListener) {
 		if (RasSrv->IsGKRouted()) {
 			m_rasListener->SetSignalPort(m_signalPort);
-			if (m_multicastListener)
+			if (m_multicastListener) {
 				m_multicastListener->SetSignalPort(m_signalPort);
-		} else
+			}
+		} else {
 			RasSrv->CloseListener(m_callSignalListener), m_callSignalListener = 0;
+		}
+	}
 
 	return m_rasListener != 0;
 }
@@ -617,13 +615,16 @@ bool RasRequester::WaitForResponse(int timeout)
 	m_timeout = timeout;
 	while (m_iterator == m_queue.end()) {
 		int passed = (int)((PTime() - m_sentTime).GetMilliSeconds());
-		if (m_timeout > passed && m_sync.Wait(m_timeout - passed))
-			if (m_timeout > 0)
+		if (m_timeout > passed && m_sync.Wait(m_timeout - passed)) {
+			if (m_timeout > 0) {
 				continue;
-			else
+			} else {
 				break;
-		if (!OnTimeout())
+			}
+		}
+		if (!OnTimeout()) {
 			break;
+		}
 	}
 	return m_iterator != m_queue.end();
 }
@@ -684,14 +685,19 @@ void RasRequester::AddReply(RasMsg *ras)
 
 
 // class RasServer
-RasServer::RasServer() : Singleton<RasServer>("RasSrv"), requestSeqNum(0),
-	acctList(NULL)
+RasServer::RasServer() : Singleton<RasServer>("RasSrv")
 {
 	SetName("RasSrv");
 
-	authList = 0;
-	sigHandler = 0;
-	broadcastListener = 0;
+	requestSeqNum = 0;
+	listeners = NULL;
+	broadcastListener = NULL;
+	sigHandler = NULL;
+	authList = NULL;
+	acctList = NULL;
+	gkClient = NULL;
+	neighbors = NULL;
+	vqueue = NULL;
 	GKRoutedSignaling = false;
 	GKRoutedH245 = false;
 	altGKs = new H225_ArrayOf_AlternateGK;
@@ -713,6 +719,7 @@ void RasServer::Stop()
 	PWaitAndSignal lock(m_deletionPreventer);
 	ForEachInContainer(handlers, mem_vfun(&RasHandler::Stop));
 	delete vqueue;	// delete virtual queues before Jobs, otherwise the jobs will wait for the queues
+	vqueue = NULL;
 	RegularJob::Stop();
 }
 
@@ -891,12 +898,18 @@ void RasServer::LoadConfig()
 	}
 #endif
 
-	listeners->LoadConfig();
-	gkClient->OnReload();
-	neighbors->OnReload();
-	authList->OnReload();
-	acctList->OnReload();
-	vqueue->OnReload();
+	if (listeners)
+		listeners->LoadConfig();
+	if (gkClient)
+		gkClient->OnReload();
+	if (neighbors)
+		neighbors->OnReload();
+	if (authList)
+		authList->OnReload();
+	if (acctList)
+		acctList->OnReload();
+	if (vqueue)
+		vqueue->OnReload();
 	Routing::Analyzer::Instance()->OnReload();
 
 	bRemoveCallOnDRQ = Toolkit::AsBool(GkConfig()->GetString(RoutedSec, "RemoveCallOnDRQ", 1));
@@ -939,27 +952,37 @@ GkInterface *RasServer::SelectInterface(const Address & addr)
 
 const GkInterface *RasServer::SelectInterface(const Address & addr) const
 {
-	return const_cast<RasServer *>(this)->SelectInterface(addr);
+	return const_cast<RasServer *>(this)->SelectInterface(addr);	// cast away const
+}
+
+RasListener * RasServer::GetRasListener(const Address & addr) const
+{
+	const GkInterface * interf = SelectInterface(addr);
+	return interf ? interf->GetRasListener() : NULL;
 }
 
 PIPSocket::Address RasServer::GetLocalAddress(const Address & addr) const
 {
-	return SelectInterface(addr)->GetRasListener()->GetPhysicalAddr(addr);
+	RasListener * listener = GetRasListener(addr);
+	return listener ? listener->GetPhysicalAddr(addr) : PIPSocket::Address(0);
 }
 
 PIPSocket::Address RasServer::GetMasqAddress(const Address & addr) const
 {
-	return SelectInterface(addr)->GetRasListener()->GetLocalAddr(addr);	
+	RasListener * listener = GetRasListener(addr);
+	return listener ? listener->GetLocalAddr(addr) : PIPSocket::Address(0);	
 }
 
 H225_TransportAddress RasServer::GetRasAddress(const Address & addr) const
 {
-	return SelectInterface(addr)->GetRasListener()->GetRasAddress(addr);
+	RasListener * listener = GetRasListener(addr);
+	return listener ? listener->GetRasAddress(addr) : H225_TransportAddress(0);
 }
 
 H225_TransportAddress RasServer::GetCallSignalAddress(const Address & addr) const
 {
-	return SelectInterface(addr)->GetRasListener()->GetCallSignalAddress(addr);
+	RasListener * listener = GetRasListener(addr);
+	return listener ? listener->GetCallSignalAddress(addr) : H225_TransportAddress(0);
 }
 
 bool RasServer::SendRas(const H225_RasMessage & rasobj, const Address & addr, WORD pt, RasListener *socket)
@@ -1146,6 +1169,20 @@ bool RasServer::LogAcctEvent(
 	return acctList->LogAcctEvent((GkAcctLogger::AcctEvent)evt,call,now);
 }
 
+PString RasServer::GetAuthInfo(
+	const PString &moduleName
+	)
+{
+	return authList->GetInfo(moduleName);
+}
+
+PString RasServer::GetAcctInfo(
+	const PString &moduleName
+	)
+{
+	return acctList->GetInfo(moduleName);
+}
+
 void RasServer::Run()
 {
 	RasMsg::Initialize();
@@ -1406,7 +1443,7 @@ template<> bool RasPDU<H225_GatekeeperRequest>::Process()
 	bool bShellSendReply = !RasSrv->IsForwardedRas(request, m_msg->m_peerAddr);
 
 	PString log;
-	PString alias(request.HasOptionalField(H225_GatekeeperRequest::e_endpointAlias)
+	PString alias((request.HasOptionalField(H225_GatekeeperRequest::e_endpointAlias) && request.m_endpointAlias.GetSize() > 0)
 		? AsString(request.m_endpointAlias[0],false) : PString(" ")
 		);
 
@@ -1495,12 +1532,24 @@ bool RegistrationRequestPDU::Process()
 
 ///////////////////////////////////////////////////////////////////////////////////////////
 // H460 support Code
-	bool supportNAT = false;
+	PBoolean supportNATOffload = false;
 	int RegPrior =0;
 	bool preemptsupport = false;
 	PBoolean preempt = false;
+	unsigned ntype = 0;
 
 #ifdef hasH460
+// Presence Support
+// Support Presence information
+#ifdef hasPresence
+	OpalOID rPreFS = OpalOID(OID3);
+	bool presencesupport = false;
+	H460_Feature * preFeature = NULL;
+#endif
+	
+// Support for NAT type detection
+// Indicates that the Endpoint registering supports Nated EPs calling it so proxying media may not be required
+	int rNaTFS = 23;    
 
 // Registration Priority and Pre-emption
 // This allows the unregistration of duplicate aliases with lower priority 
@@ -1509,6 +1558,13 @@ bool RegistrationRequestPDU::Process()
 	if (request.HasOptionalField(H225_RegistrationRequest::e_featureSet)) {
 		H460_FeatureSet fs = H460_FeatureSet(request.m_featureSet);
 
+		if (fs.HasFeature(rNaTFS)) {
+			H460_FeatureStd * natfeat = (H460_FeatureStd *)fs.GetFeature(rNaTFS);
+           if (natfeat->Contains(P2P_RemoteNAT))	  
+			      supportNATOffload = natfeat->Value(P2P_RemoteNAT);
+		   if (natfeat->Contains(P2P_NATdet))
+			      ntype = natfeat->Value(P2P_NATdet);
+		}
 		if (fs.HasFeature(rPriFS)) {
 			H460_FeatureOID * feat = (H460_FeatureOID *)fs.GetFeature(rPriFS);
 			if (feat->Contains(priorityOID)) {
@@ -1520,6 +1576,12 @@ bool RegistrationRequestPDU::Process()
                 preempt = feat->Value(PString(preemptOID));
 			}
 		}
+#ifdef hasPresence
+		if (fs.HasFeature(rPreFS)) {
+			presencesupport = true;
+			preFeature = (H460_FeatureOID *)fs.GetFeature(rPreFS);
+		}
+#endif
 	}
 #endif
 ///////////////////////////////////////////////////////////////////////////////////////////////
@@ -1575,6 +1637,24 @@ bool RegistrationRequestPDU::Process()
 			// endpoint was NOT registered and force Full Registration
 			return BuildRRJ(H225_RegistrationRejectReason::e_fullRegistrationRequired);
 		} else {
+            // If received a NAT Type update STD24 only occurs with Light RRQ
+            if (ntype > 1) {
+				if (!ep->IsNATed()) {
+				  PTRACE(2, "Std24\tEndpoint reports itself as being behind a firewall!");
+				  ep->SetNAT(true);
+				  ep->SetSupportNAT(false);
+				  ep->SetNATAddress(rx_addr);
+				}
+				PTRACE(2, "Std24\tEndPoint NAT/FW reported as being " << ep->GetEPNATTypeString((EndpointRec::EPNatTypes)ntype));
+				ep->SetEPNATType(ntype);
+            }
+
+#ifdef hasPresence
+			// If we have some presence information
+			if (presencesupport && ep->hasContact())
+				  ep->ParsePresencePDU(this->GetTag(),&ep->GetEndpointIdentifier(),*preFeature);
+#endif
+
 			// forward lightweights, too
 			if (bShellForwardRequest)
 				RasSrv->ForwardRasMsg(m_msg->m_recvRAS);
@@ -1587,15 +1667,17 @@ bool RegistrationRequestPDU::Process()
 	if (request.m_rasAddress.GetSize() == 0)
 		return BuildRRJ(H225_RegistrationRejectReason::e_invalidRASAddress);
 
-	bool nated = false, validaddress = false;
+	bool nated = false, validaddress = false, internal = false;
 	if (request.m_callSignalAddress.GetSize() >= 1) {
 		PIPSocket::Address ipaddr;
 		for (int s = 0; s < request.m_callSignalAddress.GetSize(); ++s) {
 			SignalAddr = request.m_callSignalAddress[s];
 			if (GetIPFromTransportAddr(SignalAddr, ipaddr)) {
 				validaddress = ((rx_addr == ipaddr) || IsLoopback(rx_addr));  
-				if (validaddress)
-					break;
+				if (validaddress) {
+					internal = Toolkit::Instance()->IsInternal(rx_addr);
+				    break;
+				}
 			}
 		}
 		//validaddress = PIPSocket::IsLocalHost(rx_addr.AsString());
@@ -1749,11 +1831,21 @@ bool RegistrationRequestPDU::Process()
 		return BuildRRJ(H225_RegistrationRejectReason::e_undefinedReason);
 	}
 
+#ifdef hasPresence
+	// If we have some presence information
+	if (presencesupport) {
+	   if (!ep->hasContact()) 
+		     ep->CreateContact();
+	   ep->ParsePresencePDU(this->GetTag(),&ep->GetEndpointIdentifier(),*preFeature);
+	}	
+#endif
+
 	if (nated)
 		ep->SetNATAddress(rx_addr);
 	else {
 		ep->SetNAT(false);
-		if (supportNAT)
+		ep->SetInternal(internal);
+		if (supportNATOffload)
 		  ep->SetSupportNAT(true);
 	}
 
@@ -1766,7 +1858,7 @@ bool RegistrationRequestPDU::Process()
 		//
 		BuildRCF(ep);
 		H225_RegistrationConfirm & rcf = m_msg->m_replyRAS;
-		if (supportcallingNAT && nated) {
+		if (supportcallingNAT && !internal) {
 			// tell the endpoint its translated address
 			rcf.IncludeOptionalField(H225_RegistrationConfirm::e_nonStandardData);
 		    rcf.m_nonStandardData.m_nonStandardIdentifier.SetTag(H225_NonStandardIdentifier::e_h221NonStandard);
@@ -1774,7 +1866,11 @@ bool RegistrationRequestPDU::Process()
 			t35.m_t35CountryCode = Toolkit::t35cPoland;
 			t35.m_manufacturerCode = Toolkit::t35mGnuGk;
 			t35.m_t35Extension = Toolkit::t35eNATTraversal;
+		  // if the client is NAT or you are forcing ALL registration to use a keepAlive TCP socket
+		  if ((nated) || Toolkit::AsBool(Kit->Config()->GetString(RoutedSec, "ForceNATKeepAlive", "0")))
 			rcf.m_nonStandardData.m_data = "NAT=" + rx_addr.AsString();
+		  else  // Be careful as some public IP's may block TCP but not UDP resulting in an incorrect NAT test result.
+		    rcf.m_nonStandardData.m_data = "NoNAT";
 		}
 
 #ifdef hasH460
@@ -1788,6 +1884,42 @@ bool RegistrationRequestPDU::Process()
 			  gd.SetSize(lPos+1);
 			  gd[lPos] = pre;
 			}
+
+#ifdef hasPresence	    
+		   // If presence support
+		   if (presencesupport) {
+	          H460_FeatureOID presence = H460_FeatureOID(rPreFS);
+			  ep->BuildPresencePDU(rcf.GetTag(),*preFeature);
+			  PINDEX lPos = gd.GetSize();
+			  gd.SetSize(lPos+1);
+			  gd[lPos] = presence;
+	       }
+#endif
+		   // if we support NAT notify the client they are behind a NAT or to test for NAT
+		   // send off a request to test the client NAT type with a STUN Server
+			if (supportcallingNAT && supportNATOffload && (ep->GetEPNATType() == 0)) {  
+			  H460_FeatureStd natfs = H460_FeatureStd(rNaTFS);
+              natfs.Add(P2P_IsNAT,H460_FeatureContent(nated));
+
+			  // if detect behind a public NAT or on a public IP (test to see if behind firewall)
+			  if (!rx_addr.IsRFC1918()) { 
+                PIPSocket::Address m_ip;
+			    WORD m_port;
+			    PString stun = Kit->Config()->GetString(RoutedSec, "STUNServer", "");
+
+				if (!stun && GetTransportAddress(stun, GK_DEF_STUN_PORT, m_ip, m_port)) {
+					H323TransportAddress stunaddr = H323TransportAddress(m_ip,m_port);
+					natfs.Add(P2P_STUNAddr,H460_FeatureContent(stunaddr));
+					natfs.Add(P2P_ProxyNAT,H460_FeatureContent(ep->HasNATProxy()));
+				}
+			  }
+			  natfs.Add(P2P_DetRASAddr,H460_FeatureContent(H323TransportAddress(request.m_rasAddress[0])));
+
+		      PINDEX lPos = gd.GetSize();
+			  gd.SetSize(lPos+1);
+			  gd[lPos] = natfs;
+			}
+
 			if (gd.GetSize() > 0)		  
 				rcf.IncludeOptionalField(H225_RegistrationConfirm::e_genericData);
 #endif
@@ -2120,12 +2252,12 @@ bool AdmissionRequestPDU::Process()
 		authData.m_dialedNumber = pExistingCallRec->GetDialedNumber();
 		
 	if (authData.m_dialedNumber.IsEmpty()) {
-		if (!answer && hasDestInfo)
+		if (!answer && hasDestInfo) {
 			authData.m_dialedNumber = GetBestAliasAddressString(
 				request.m_destinationInfo, false,
 				AliasAddressTagMask(H225_AliasAddress::e_dialedDigits)
-					| AliasAddressTagMask(H225_AliasAddress::e_partyNumber)
-				);
+					| AliasAddressTagMask(H225_AliasAddress::e_partyNumber));
+		}
 	}
 		
 	if (hasDestInfo) { // apply rewriting rules
@@ -2137,22 +2269,24 @@ bool AdmissionRequestPDU::Process()
 				| AliasAddressTagMask(H225_AliasAddress::e_partyNumber)
 			);
 
-     	if(in_rewrite_source.IsEmpty() && request.m_srcInfo.GetSize() > 0) {
+     	if (in_rewrite_source.IsEmpty() && request.m_srcInfo.GetSize() > 0) {
         	in_rewrite_source = GetBestAliasAddressString(request.m_srcInfo, false,
 				AliasAddressTagMask(H225_AliasAddress::e_h323_ID), 
 				AliasAddressTagMask(H225_AliasAddress::e_dialedDigits)
-					| AliasAddressTagMask(H225_AliasAddress::e_partyNumber)
-				);
+					| AliasAddressTagMask(H225_AliasAddress::e_partyNumber));
 		}
 
-	 	if (!in_rewrite_source.IsEmpty())
+	 	if (!in_rewrite_source.IsEmpty()) {
 	 		if (Kit->GWRewriteE164(in_rewrite_source, true, request.m_destinationInfo[0])
-				&& !RasSrv->IsGKRouted())
+				&& !RasSrv->IsGKRouted()) {
 				aliasesChanged = true;
+			}
+		}
 
 		// Normal rewriting
-		if (Kit->RewriteE164(request.m_destinationInfo[0]) && !RasSrv->IsGKRouted())
+		if (Kit->RewriteE164(request.m_destinationInfo[0]) && !RasSrv->IsGKRouted()) {
 			aliasesChanged = true;
+		}
 	}
 
 	destinationString = hasDestInfo ? AsString(request.m_destinationInfo) :
@@ -2164,8 +2298,9 @@ bool AdmissionRequestPDU::Process()
 	authData.m_callLinkage = GetCallLinkage(authData);
 	
 	if (!RasSrv->ValidatePDU(*this, authData)) {
-		if (authData.m_rejectReason < 0)
+		if (authData.m_rejectReason < 0) {
 			authData.m_rejectReason = H225_AdmissionRejectReason::e_securityDenial;
+		}
 		return BuildReply(authData.m_rejectReason);
 	}
 
@@ -2177,8 +2312,9 @@ bool AdmissionRequestPDU::Process()
 		PTRACE(2, "RAS\tARQ destination set to " << authData.m_calledStationId);
 		hasDestInfo = true;
 		destinationString = AsString(request.m_destinationInfo);
-		if (!RasSrv->IsGKRouted())
+		if (!RasSrv->IsGKRouted()) {
 			aliasesChanged = true;
+		}
 	}
 	
 	if (RasSrv->IsGKRouted() && answer && !pExistingCallRec) {
@@ -2186,15 +2322,19 @@ bool AdmissionRequestPDU::Process()
 			bReject = true;
 			if (request.HasOptionalField(H225_AdmissionRequest::e_srcCallSignalAddress)) {
 				PIPSocket::Address ipaddress;
-				if (GetIPFromTransportAddr(request.m_srcCallSignalAddress, ipaddress))
+				if (GetIPFromTransportAddr(request.m_srcCallSignalAddress, ipaddress)) {
 					bReject = !RasSrv->IsForwardedMessage(0, ipaddress);
+				}
 			}
-			if (bReject)
+			if (bReject) {
 				return BuildReply(H225_AdmissionRejectReason::e_routeCallToGatekeeper);
+			}
 		}
 	}
 
+	CallRec::NatStrategy natoffloadsupport = CallRec::e_natUnknown;
 #ifdef hasH460
+	bool natsupport = false;
     bool QoSReporting = false;
 	bool vendorInfo = false;
 	PString m_vendor,m_version = PString();
@@ -2202,6 +2342,15 @@ bool AdmissionRequestPDU::Process()
 		H225_ArrayOf_GenericData & data = request.m_genericData;
 		for (PINDEX i =0; i < data.GetSize(); i++) {
           H460_Feature & feat = (H460_Feature &)data[i];
+          /// Std24
+		  if (feat.GetFeatureID() == H460_FeatureID(24)) {
+		     natsupport = true;
+			 H460_FeatureStd & std24 = (H460_FeatureStd &)feat;
+			 if (std24.Contains(P2P_NATInstruct)) {
+			   unsigned natstat = std24.Value(P2P_NATInstruct);
+			   natoffloadsupport = (CallRec::NatStrategy)natstat;
+			 }
+		  }
 		  /// Vendor Information
   		  if (feat.GetFeatureID() == H460_FeatureID(OpalOID(OID9))) 
 		     vendorInfo = true;
@@ -2213,8 +2362,7 @@ bool AdmissionRequestPDU::Process()
 #endif
 
 	//
-	// Bandwidth
-	// and GkManager admission
+	// Bandwidth check
 	//
 	int BWRequest = request.m_bandWidth.GetValue();
 	// hack for Netmeeting 3.0x
@@ -2223,11 +2371,13 @@ bool AdmissionRequestPDU::Process()
 	// check if it is the first arrived ARQ
 	if (pExistingCallRec) {
 		// request more bandwidth?
-		if (BWRequest > pExistingCallRec->GetBandwidth())
-			if (CallTbl->GetAdmission(BWRequest, pExistingCallRec))
+		if (BWRequest > pExistingCallRec->GetBandwidth()) {
+			if (CallTbl->GetAdmission(BWRequest, pExistingCallRec)) {
 				pExistingCallRec->SetBandwidth(BWRequest);
-			else
+			} else {
 				bReject = true;
+			}
+		}
 	} else {
 		bReject = (!CallTbl->GetAdmission(BWRequest));
 	}
@@ -2307,6 +2457,25 @@ bool AdmissionRequestPDU::Process()
 
 	}
 
+	if (hasDestInfo && CalledEP && RequestingEP != CalledEP && !arq.GetRoutes().empty() && !arq.GetRoutes().front().m_destOutNumber.IsEmpty()) {
+		for (PINDEX i = 0; i < request.m_destinationInfo.GetSize(); ++i)
+			if (request.m_destinationInfo[i].GetTag() == H225_AliasAddress::e_dialedDigits) {
+				H323SetAliasAddress(arq.GetRoutes().front().m_destOutNumber, request.m_destinationInfo[i], request.m_destinationInfo[i].GetTag());
+				aliasesChanged = aliasesChanged || RasSrv->IsGKRouted();
+			} else if (request.m_destinationInfo[i].GetTag() == H225_AliasAddress::e_partyNumber) {
+				H225_PartyNumber &partyNumber = request.m_destinationInfo[i];
+				if (partyNumber.GetTag() == H225_PartyNumber::e_e164Number) {
+					H225_PublicPartyNumber &number = partyNumber;
+					number.m_publicNumberDigits = arq.GetRoutes().front().m_destOutNumber;
+					aliasesChanged = aliasesChanged || RasSrv->IsGKRouted();
+				} else if (partyNumber.GetTag() == H225_PartyNumber::e_privateNumber) {
+					H225_PrivatePartyNumber &number = partyNumber;
+					number.m_privateNumberDigits = arq.GetRoutes().front().m_destOutNumber;
+					aliasesChanged = aliasesChanged || RasSrv->IsGKRouted();
+				}
+			}
+	}
+	
 	if (pExistingCallRec) {
 		// duplicate or answer ARQ
 		PTRACE(3, "GK\tACF: found existing call no " << pExistingCallRec->GetCallNumber());
@@ -2333,6 +2502,11 @@ bool AdmissionRequestPDU::Process()
 		if (toParent)
 			pCallRec->SetToParent(true);
 
+		if (natoffloadsupport == CallRec::e_natNoassist) { // If no assistance then No NAT type
+			PTRACE(4,"RAS\tNAT Type reset to none"); 
+			pCallRec->SetNATType(0);
+		}
+
 		pCallRec->SetNewRoutes(arq.GetRoutes());
 		
 		if (authData.m_callDurationLimit > 0)
@@ -2353,6 +2527,37 @@ bool AdmissionRequestPDU::Process()
 			pCallRec->SetAccessTokens(acf.m_cryptoTokens);
 		CallTbl->Insert(pCallRec);
 
+#ifdef hasH460
+		// Std24 proxy offload. See if the media can go direct.
+		if (natoffloadsupport == CallRec::e_natUnknown) 
+		    if (!pCallRec->NATOffLoad(answer,natoffloadsupport))
+			 if (natoffloadsupport == CallRec::e_natFailure) {
+				PTRACE(2,"RAS\tWarning: NAT Media Failure detected " << (unsigned)request.m_callReferenceValue);
+				return BuildReply(H225_AdmissionRejectReason::e_noRouteToDestination);
+			 }
+
+			
+		// If not required disable the proxy support function for this call
+		if (pCallRec->GetProxyMode() != CallRec::ProxyDisabled &&
+			(natoffloadsupport == CallRec::e_natLocalMaster || 
+				natoffloadsupport == CallRec::e_natRemoteMaster ||
+				natoffloadsupport == CallRec::e_natNoassist ||
+			(!pCallRec->SingleGatekeeper() && natoffloadsupport == CallRec::e_natRemoteProxy))) {
+					PTRACE(4,"RAS\tNAT Proxy disabled due to offload support"); 
+					pCallRec->SetProxyMode(CallRec::ProxyDisabled);
+		}
+
+		pCallRec->SetNATStrategy(natoffloadsupport);
+		PTRACE(4,"RAS\tNAT strategy for Call No: " << pCallRec->GetCallNumber() << 
+					" set to " << pCallRec->GetNATOffloadString(natoffloadsupport));
+
+		if (natoffloadsupport == CallRec::e_natNoassist) { // If no assistance then No NAT type
+			PTRACE(4,"RAS\tNAT Type reset to none"); 
+			pCallRec->SetNATType(0);
+		}
+
+		pCallRec->GetRemoteInfo(m_vendor,m_version);
+#endif
 		// Put rewriting information into call record
 		pCallRec->SetInboundRewriteId(in_rewrite_source);
 		pCallRec->SetOutboundRewriteId(out_rewrite_source);
@@ -2394,6 +2599,14 @@ bool AdmissionRequestPDU::Process()
 	if (!answer) {	
 	   PINDEX lastPos = 0;
        H225_ArrayOf_GenericData & data = acf.m_genericData;
+   /// Std24 Nat Traversal
+	   if (natsupport) {
+	     H460_FeatureStd fs = H460_FeatureStd(24);
+	     fs.Add(P2P_NATInstruct,H460_FeatureContent((int)natoffloadsupport,8));
+         lastPos++;
+	     data.SetSize(lastPos);
+	     data[lastPos-1] = fs;   
+	   }
    /// OID9 Vendor Information
 	   if (vendorInfo && !m_vendor.IsEmpty()) {
 	      H460_FeatureOID fs = H460_FeatureOID(OID9);
@@ -2538,7 +2751,7 @@ template<> bool RasPDU<H225_DisengageRequest>::Process()
 		rsn = H225_DisengageRejectReason::e_notRegistered;
 	}
 
-#if hasH460
+#ifdef hasH460
 	if (request.HasOptionalField(H225_DisengageRequest::e_genericData)) {  
 		H225_ArrayOf_GenericData & data = request.m_genericData;
 		for (PINDEX i =0; i < data.GetSize(); i++) {
@@ -2674,6 +2887,22 @@ template<> bool RasPDU<H225_LocationRequest>::Process()
 				  for (PINDEX i = 0; i < locdata.GetSize(); i++) {
 				   
 				  H460_Feature & feat = (H460_Feature &)locdata[i];
+                   /// Std24 NAT Traversal
+				   if (feat.GetFeatureID() == H460_FeatureID(24)) {
+				     H460_FeatureStd std24 = H460_FeatureStd(24);
+					  if (WantedEndPoint->IsNATed()) {
+	                    std24.Add(P2P_IsNAT,H460_FeatureContent(true));
+					    std24.Add(P2P_NATdet,H460_FeatureContent(WantedEndPoint->GetEPNATType(),8));
+  					    std24.Add(P2P_ProxyNAT,H460_FeatureContent(WantedEndPoint->HasNATProxy()));
+						std24.Add(P2P_SourceAddr,H460_FeatureContent(WantedEndPoint->GetNATIP().AsString()));
+					  } else {
+						std24.Add(P2P_RemoteNAT,H460_FeatureContent(WantedEndPoint->SupportNAT()));
+						std24.Add(P2P_MustProxy,H460_FeatureContent(WantedEndPoint->IsInternal()));
+				      }
+					  lastPos++;
+					  data.SetSize(lastPos);
+				      data[lastPos-1] = std24;
+				   }
 				   /// OID9 Vendor Interoperability
 			       if (feat.GetFeatureID() == H460_FeatureID(OpalOID(OID9))) {
 	                  PString m_vendor,m_version = PString();
@@ -2761,7 +2990,7 @@ template<> bool RasPDU<H225_InfoRequestResponse>::Process()
 			call = CallTbl->FindBySignalAdr(request.m_callSignalAddress[0]);
 		if (call)
 			call->Update(request);
-#if hasH460
+#ifdef hasH460
 		if (call && request.HasOptionalField(H225_InfoRequestResponse::e_genericData)) {  
 			H225_ArrayOf_GenericData & data = request.m_genericData;
 			for (PINDEX i =0; i < data.GetSize(); i++) {
