@@ -17,7 +17,7 @@
 //////////////////////////////////////////////////////////////////
 
 #ifndef RASSRV_H
-#define RASSRV_H "@(#) $Id: RasSrv.h,v 1.45 2007/11/19 20:15:34 willamowius Exp $"
+#define RASSRV_H "@(#) $Id: RasSrv.h,v 1.48 2008/08/27 12:37:54 willamowius Exp $"
 
 #include <vector>
 #include <list>
@@ -102,6 +102,7 @@ public:
 
 	GkInterface *SelectInterface(const Address &);
 	const GkInterface *SelectInterface(const Address &) const;
+	RasListener * GetRasListener(const Address & addr) const;
 	Address GetLocalAddress(const Address &) const;
 	Address GetMasqAddress(const Address &) const;
 	H225_TransportAddress GetRasAddress(const Address &) const;
@@ -159,6 +160,9 @@ public:
 	{
 		return authList->Validate(setup, authData);
 	}
+	
+	GkAcctLoggerList * GetAcctList() { return acctList; }
+	GkAuthenticatorList * GetAuthList() { return authList; }
 
 	bool LogAcctEvent(
 		int evt,
@@ -166,6 +170,14 @@ public:
 		time_t now = 0
 		);
 	
+	PString GetAuthInfo(
+		const PString &moduleName
+		);
+
+	PString GetAcctInfo(
+		const PString &moduleName
+		);
+
 	template<class RAS> bool IsForwardedRas(const RAS & ras, const Address & addr) const
 	{
 		return IsForwardedMessage(ras.HasOptionalField(RAS::e_nonStandardData) ? &ras.m_nonStandardData : 0, addr);
