@@ -10,7 +10,7 @@
 //////////////////////////////////////////////////////////////////
 
 #ifndef RASTBL_H
-#define RASTBL_H "@(#) $Id: RasTbl.h,v 1.139.2.1 2008/04/19 00:06:59 willamowius Exp $"
+#define RASTBL_H "@(#) $Id: RasTbl.h,v 1.147 2008/11/16 12:54:16 willamowius Exp $"
 
 #include <list>
 #include <map>
@@ -19,7 +19,8 @@
 #include "singleton.h"
 #include "h225.h"
 #include "sigmsg.h"
-#include "pwlib_compat.h"
+#include "config.h"
+#include "h323util.h"
 
 
 #if (_MSC_VER >= 1200)
@@ -62,6 +63,9 @@ private:
 	T *pt;
 };
 
+#ifdef hasPresence
+class H323Contact;
+#endif
 class EndpointRec
 {
 public:
@@ -81,6 +85,7 @@ public:
 	// public interface to access EndpointRec
 	H225_TransportAddress GetRasAddress() const;
 	H225_TransportAddress GetCallSignalAddress() const;
+	PIPSocket::Address GetIP() const;
 	H225_EndpointIdentifier GetEndpointIdentifier() const;
 	H225_ArrayOf_AliasAddress GetAliases() const;
 	H225_EndpointType GetEndpointType() const;
@@ -90,6 +95,7 @@ public:
 	CallSignalSocket *GetSocket();
 
 	int GetCallTypeOfNumber(bool called = true) const { return called ? m_calledTypeOfNumber : m_callingTypeOfNumber; }
+	int GetCallPlanOfNumber(bool called = true) const { return called ? m_calledPlanOfNumber : m_callingPlanOfNumber; }
 	int GetProxyType() const { return m_proxy; }
 
 	virtual void SetRasAddress(const H225_TransportAddress &);
@@ -155,6 +161,23 @@ public:
 	void SetNATAddress(const PIPSocket::Address &);
 	void SetSocket(CallSignalSocket *);
 	void SetSupportNAT(bool support);
+
+	enum EPNatTypes {
+            NatUnknown,
+            NatOpen,
+            NatCone,
+            NatRestricted,
+            NatPortRestricted,
+            NatSymmetric,
+            FirewallSymmetric,
+            NatBlocked,
+            NatPartialBlocked
+       };
+
+	void SetEPNATType(int nattype) {m_epnattype = (EPNatTypes)nattype; }
+	void SetNATProxy(PBoolean support) {m_natproxy = support; }
+	void SetInternal(PBoolean internal) { m_internal = internal; }
+
 	void SetPriority(int priority) { m_registrationPriority = priority; };
 	void SetPreemption(bool support) { m_registrationPreemption = support; };
 	void SetAssignedGatekeeper(const H225_AlternateGK & gk) { m_assignedGatekeeper = gk; };
@@ -168,13 +191,27 @@ public:
 	bool IsUsed() const;
 	bool IsUpdated(const PTime *) const;
 	bool IsNATed() const;
+	bool SupportNAT() const;
 
+	bool HasNATProxy() const;
+	bool IsInternal() const;
+	bool IsRemote() const;
+	int GetEPNATType() {return (int)m_epnattype; }
+	static PString GetEPNATTypeString(EPNatTypes nat);
+	bool SupportPreemption() const { return m_registrationPreemption; }
 	H225_AlternateGK GetAssignedGatekeeper() { return m_assignedGatekeeper; }
 
 	int  Priority() const { return m_registrationPriority; }
 	bool HasNATSocket() const;
 	PTime GetUpdatedTime() const;
 
+#ifdef hasPresence
+	bool hasContact();
+	void CreateContact();
+	void ParsePresencePDU(unsigned msgtag, const H225_EndpointIdentifier * id, const H225_FeatureDescriptor & pdu);
+	void BuildPresencePDU(unsigned msgtag, H225_FeatureDescriptor & pdu);
+#endif
+
 	/** If this Endpoint would be register itself again with all the same data
 	 * how would this RRQ would look like? May be implemented with a
 	 * built-together-RRQ, but for the moment a stored RRQ.
@@ -247,8 +284,6 @@ private:
 	/// Load general endpoint settings from the config
 	void LoadEndpointConfig();
 	void AddPrefixCapacities(const PString & prefixes);
-	void ParseTranslationMap(std::map<unsigned, unsigned> & cause_map, const PString & ini);
-
 
 	EndpointRec();
 	EndpointRec(const EndpointRec &);
@@ -277,7 +312,7 @@ protected:
 	mutable PMutex m_usedLock;
 
 	PTime m_updatedTime;
-	bool m_fromParent, m_nat, m_natsupport;
+	bool m_fromParent, m_nat;
 	PIPSocket::Address m_natip;
 	CallSignalSocket *m_natsocket;
 	/// permanent (preconfigured) endpoint flag
@@ -291,6 +326,7 @@ protected:
 	/// capacity per prefix (regex)
 	list<pair<string, int> > m_prefixCapacities;
 	int m_calledTypeOfNumber, m_callingTypeOfNumber;
+	int m_calledPlanOfNumber, m_callingPlanOfNumber;
 	/// Proxy Type
 	int m_proxy;
 	/// Registration Priority Number
@@ -303,6 +339,14 @@ protected:
 	/// cause code translation
 	std::map<unsigned, unsigned> m_receivedCauseMap;
 	std::map<unsigned, unsigned> m_sentCauseMap;
+
+	EPNatTypes m_epnattype;
+	bool m_natsupport, m_natproxy, m_internal, m_remote;
+
+#ifdef hasPresence
+	H323Contact * m_contact;
+#endif
+
 };
 
 typedef EndpointRec::Ptr endptr;
@@ -425,7 +469,7 @@ public:
 	endptr FindOZEPBySignalAdr(const H225_TransportAddress &) const;
 	endptr FindByAliases(const H225_ArrayOf_AliasAddress & alias) const;
 	endptr FindFirstEndpoint(const H225_ArrayOf_AliasAddress & alias);
-	void FindEndpoint(
+	bool FindEndpoint(
 		const H225_ArrayOf_AliasAddress &aliases,
 		bool roundRobin,
 		bool searchOuterZone,
@@ -472,7 +516,7 @@ private:
 	}
 
 	endptr InternalFindFirstEP(const H225_ArrayOf_AliasAddress & alias, std::list<EndpointRec *> *ListToBeFound);
-	void InternalFindEP(const H225_ArrayOf_AliasAddress & alias, std::list<EndpointRec *> *ListToBeFound, bool roundrobin, std::list<Routing::Route> &routes);
+	bool InternalFindEP(const H225_ArrayOf_AliasAddress & alias, std::list<EndpointRec *> *ListToBeFound, bool roundrobin, std::list<Routing::Route> &routes);
 
 	void GenerateEndpointId(H225_EndpointIdentifier &);
 	void GenerateAlias(H225_ArrayOf_AliasAddress &, const H225_EndpointIdentifier &) const;
@@ -578,6 +622,46 @@ public:
 		PIPSocket::Address& calledPartyNATIP
 		) const;
 
+	// Override the calculated NAT type
+	void SetNATType(int newNatType) { m_nattype = newNatType; }
+
+	/** Can the call party be used to bypass proxy for NAT
+	*/
+	enum NatStrategy {
+		e_natUnknown,
+		e_natNoassist,
+		e_natLocalMaster,
+	    e_natRemoteMaster,
+		e_natLocalProxy,
+	    e_natRemoteProxy,
+		e_natFullProxy,
+		e_natSameNAT,
+		e_natFailure = 100
+	};
+
+	/** Return whether a call can be offloaded from having to proxy
+	  */
+	bool NATOffLoad(bool iscalled, NatStrategy & natinst);
+
+	/** Get String representation of the NATStrategy */
+	PString GetNATOffloadString(NatStrategy nat);
+
+	/** Get the NATStrategy Default is Unknown */
+	NatStrategy GetNATStrategy() { return m_natstrategy; }
+
+	/** Set the NATStrategy */
+	void SetNATStrategy(NatStrategy strategy) { m_natstrategy = strategy; }
+
+
+	/** Return whether the endpoints are registered at the same gatekeeper so
+	    only 1 gatekeeper is involved in the call
+	  */
+	bool SingleGatekeeper();
+
+	/** Return remote party device information
+	  */
+	bool GetRemoteInfo(PString & vendor, PString & version);
+
 	/** @return
 	    Current proxy mode flag (see #ProxyMode enum#).
 	*/
@@ -922,9 +1006,8 @@ public:
 	/// update IRR timers
 	void Update(const H225_InfoRequestResponse & irr);
 
-	void SetNewRoutes(
-		const std::list<Routing::Route> &routes
-		);
+	void ClearRoutes();
+	void SetNewRoutes(const std::list<Routing::Route> &routes);
 	const std::list<Routing::Route> &GetNewRoutes() const { return m_newRoutes; }
 	const std::list<Routing::Route> &GetFailedRoutes() const { return m_failedRoutes; }
 	bool MoveToNextRoute();
@@ -1045,6 +1128,7 @@ private:
 	int m_usedCount;
 	mutable PTimedMutex m_usedLock, m_sockLock;
 	int m_nattype;
+	NatStrategy m_natstrategy;
 
 	/// unregistered caller NAT'd
 	bool m_unregNAT;
@@ -1225,6 +1309,14 @@ inline H225_TransportAddress EndpointRec::GetCallSignalAddress() const
 	return m_callSignalAddress;
 }
 
+inline PIPSocket::Address EndpointRec::GetIP() const
+{
+	PWaitAndSignal lock(m_usedLock);
+	PIPSocket::Address addr;
+    GetIPFromTransportAddr(m_callSignalAddress,addr);
+	return addr;
+}
+
 inline H225_EndpointIdentifier EndpointRec::GetEndpointIdentifier() const
 {
 	PWaitAndSignal lock(m_usedLock);
@@ -1290,6 +1382,46 @@ inline bool EndpointRec::IsNATed() const
 	return m_nat;
 }
 
+inline bool EndpointRec::SupportNAT() const
+{
+	return m_natsupport;
+}
+
+inline bool EndpointRec::HasNATProxy() const
+{
+	return m_natproxy;
+}
+
+inline bool EndpointRec::IsInternal() const
+{
+	return m_internal;
+}
+
+inline bool EndpointRec::IsRemote() const
+{
+	return m_remote;
+}
+
+inline PString EndpointRec::GetEPNATTypeString(EPNatTypes nat)
+{
+  static const char * const Names[9] = {
+    "Unknown NAT",
+    "Open NAT",
+    "Cone NAT",
+    "Restricted NAT",
+    "Port Restricted NAT",
+    "Symmetric NAT",
+    "Symmetric Firewall",
+    "Blocked",
+    "Partially Blocked"
+  };
+
+  if (nat < 9)
+    return Names[nat];
+  
+  return PString((unsigned)nat);
+}
+
 inline PIPSocket::Address EndpointRec::GetNATIP() const
 {
 	PWaitAndSignal lock(m_usedLock);
