@@ -32,8 +32,8 @@
 #include "GkStatus.h"
 #include "sigmsg.h"
 #include "Routing.h"
-#include "pwlib_compat.h"
 #include "gksql.h"
+#include "config.h"
 
 using std::string;
 using std::vector;
@@ -62,7 +62,7 @@ Route::Route() : m_proxyMode(CallRec::ProxyDetect), m_flags(0)
 Route::Route(
 	const endptr &destEndpoint
 	) : m_destAddr(destEndpoint->GetCallSignalAddress()), m_destEndpoint(destEndpoint),
-	m_proxyMode(CallRec::ProxyDetect), m_flags(0), m_destNumber("")
+	m_proxyMode(CallRec::ProxyDetect), m_flags(0), m_destNumber(""), m_destOutNumber("")
 {
 	Toolkit::Instance()->SetRerouteCauses(m_rerouteCauses);
 }
@@ -70,7 +70,7 @@ Route::Route(
 Route::Route(
 	const PString &policyName,
 	const H225_TransportAddress &destAddr
-	) : m_destAddr(destAddr), m_policy(policyName), m_proxyMode(CallRec::ProxyDetect), m_flags(0), m_destNumber("")
+	) : m_destAddr(destAddr), m_policy(policyName), m_proxyMode(CallRec::ProxyDetect), m_flags(0), m_destNumber(""), m_destOutNumber("")
 {
 	Toolkit::Instance()->SetRerouteCauses(m_rerouteCauses);
 }
@@ -80,7 +80,7 @@ Route::Route(
 	const PIPSocket::Address &destIpAddr,
 	WORD destPort
 	) : m_destAddr(SocketToH225TransportAddr(destIpAddr, destPort)),
-	m_policy(policyName), m_proxyMode(CallRec::ProxyDetect), m_flags(0), m_destNumber("")
+	m_policy(policyName), m_proxyMode(CallRec::ProxyDetect), m_flags(0), m_destNumber(""), m_destOutNumber("")
 {
 	Toolkit::Instance()->SetRerouteCauses(m_rerouteCauses);
 }
@@ -88,7 +88,8 @@ Route::Route(
 PString Route::AsString() const
 {
 	return AsDotString(m_destAddr) + " (policy: " + m_policy + ", proxy: "
-		+ PString(m_proxyMode) + ", flags: " + PString(m_flags) + ", DialedNumber: " + m_destNumber + ")";
+		+ PString(m_proxyMode) + ", flags: " + PString(m_flags) + ", Called-Station-Id: " + m_destNumber
+		+ ", Called-Station-Id-Out: " + m_destOutNumber + ")";
 }
 
 bool Route::IsFailoverActive(
@@ -424,6 +425,8 @@ protected:
 
 	virtual bool FindByAliases(RoutingRequest &, H225_ArrayOf_AliasAddress &);
 	virtual bool FindByAliases(LocationRequest &, H225_ArrayOf_AliasAddress &);
+	virtual bool FindByAliases(SetupRequest &, H225_ArrayOf_AliasAddress &);
+	virtual bool FindByAliases(AdmissionRequest &, H225_ArrayOf_AliasAddress &);
 	
 private:
 	bool roundRobin;
@@ -477,9 +480,45 @@ bool InternalPolicy::FindByAliases(
 {
 	// do not apply round robin selection for Location ReQuests
 	list<Route> routes;
-	RegistrationTable::Instance()->FindEndpoint(
-		aliases, false, true, routes
-		);
+	if (RegistrationTable::Instance()->FindEndpoint(aliases, false, true, routes))
+		request.SetRejectReason(H225_LocationRejectReason::e_resourceUnavailable);
+		
+	list<Route>::iterator i = routes.begin();
+	while (i != routes.end()) {
+		i->m_policy = m_name;
+		request.AddRoute(*i++);
+	}
+	return !routes.empty();
+}
+
+bool InternalPolicy::FindByAliases(
+	SetupRequest& request,
+	H225_ArrayOf_AliasAddress & aliases
+	)
+{
+	// do not apply round robin selection for Location ReQuests
+	list<Route> routes;
+	if (RegistrationTable::Instance()->FindEndpoint(aliases, false, true, routes))
+		request.SetRejectReason(H225_ReleaseCompleteReason::e_gatewayResources);
+		
+	list<Route>::iterator i = routes.begin();
+	while (i != routes.end()) {
+		i->m_policy = m_name;
+		request.AddRoute(*i++);
+	}
+	return !routes.empty();
+}
+
+bool InternalPolicy::FindByAliases(
+	AdmissionRequest& request,
+	H225_ArrayOf_AliasAddress & aliases
+	)
+{
+	// do not apply round robin selection for Location ReQuests
+	list<Route> routes;
+	if (RegistrationTable::Instance()->FindEndpoint(aliases, false, true, routes))
+		request.SetRejectReason(H225_AdmissionRejectReason::e_resourceUnavailable);
+		
 	list<Route>::iterator i = routes.begin();
 	while (i != routes.end()) {
 		i->m_policy = m_name;
@@ -498,7 +537,7 @@ public:
 
 private:
 	// override from class Policy
-	virtual bool IsActive();
+	virtual bool IsActive() const;
 
 	virtual bool OnRequest(AdmissionRequest &);
 	virtual bool OnRequest(LocationRequest &);
@@ -514,9 +553,9 @@ ParentPolicy::ParentPolicy()
 	m_name = "Parent";
 }
 
-bool ParentPolicy::IsActive()
+bool ParentPolicy::IsActive() const
 {
-	return m_gkClient->IsRegistered();
+	return m_gkClient ? m_gkClient->IsRegistered() : false;
 }
 
 bool ParentPolicy::OnRequest(AdmissionRequest & arq_obj)
@@ -597,8 +636,7 @@ VirtualQueue::~VirtualQueue()
 	int numrequests = m_pendingRequests.size();
 	if( numrequests ) {
 		PTRACE(1,"VQueue\tDestroying virtual queue with "
-			<< numrequests << " pending requests"
-			);
+			<< numrequests << " pending requests");
 	}
 	RouteRequests::iterator i = m_pendingRequests.begin();
 	while (i != m_pendingRequests.end()) {
@@ -665,8 +703,9 @@ void VirtualQueue::OnReload()
 		}
 	}
 	
-	if( !m_active )
+	if( !m_active ) {
 		PTRACE(2,"VQueue\t(CTI) Virtual queues disabled - no virtual queues configured");
+	}
 }
 
 bool VirtualQueue::SendRouteRequest(
@@ -711,9 +750,9 @@ bool VirtualQueue::SendRouteRequest(
 		msg += PString("|") + calledip;
 		msg += PString(";");
 		// signal RouteRequest to the status line only once
-		if( duprequest )
+		if( duprequest ) {
 			PTRACE(4, "VQueue\tDuplicate request: "<<msg);
-		else {
+		} else {
 			PTRACE(2, msg);
 			GkStatus::Instance()->SignalStatus(msg + "\r\n", STATUS_TRACE_LEVEL_ROUTEREQ);
 		}
@@ -724,10 +763,10 @@ bool VirtualQueue::SendRouteRequest(
 		m_listMutex.Wait();
 		m_pendingRequests.remove(r);
 		m_listMutex.Signal();
-		if( !result )
+		if( !result ) {
 			PTRACE(5,"VQueue\tRoute request (EPID: " << r->m_callingEpId
-				<< ", CRV=" << r->m_crv << ") timed out"
-				);
+				<< ", CRV=" << r->m_crv << ") timed out");
+		}
 		delete r;
 	}
 	return result;
@@ -793,20 +832,22 @@ bool VirtualQueue::RouteToAlias(
 			r->m_sync.Signal();
 			if (!foundrequest) {
 				foundrequest = true;
-				if (!reject)
+				if (!reject) {
 					PTRACE(2,"VQueue\tRoute request (EPID:" << callingEpId
 						<< ", CRV=" << crv << ") accepted by agent " << AsString(agent));
-				else
+				} else {
 					PTRACE(2,"VQueue\tRoute request (EPID:" << callingEpId
 						<< ", CRV=" << crv << ") rejected");
+				}
 			}
 		}
 		++i;
 	}
 	
-	if( !foundrequest )
+	if( !foundrequest ) {
 		PTRACE(4, "VQueue\tPending route request (EPID:" << callingEpId
 			<< ", CRV=" << crv << ") not found - ignoring RouteToAlias / RouteToGateway command");
+	}
 	
 	return foundrequest;
 }
@@ -919,7 +960,7 @@ VirtualQueuePolicy::VirtualQueuePolicy()
 
 bool VirtualQueuePolicy::IsActive() const
 {
-	return m_vqueue->IsActive();
+	return m_vqueue ? m_vqueue->IsActive() : false;
 }
 
 bool VirtualQueuePolicy::OnRequest(AdmissionRequest & request)
@@ -973,7 +1014,7 @@ bool VirtualQueuePolicy::OnRequest(LocationRequest & request)
 {
 	bool reject = false;
 	if (H225_ArrayOf_AliasAddress *aliases = request.GetAliases()) {
-		const PString vq(AsString((*aliases)[0], FALSE));
+		const PString vq(AsString((*aliases)[0], false));
 		if (m_vqueue->IsDestinationVirtualQueue(vq)) {
 			H225_LocationRequest & lrq = request.GetRequest();
 			PTRACE(5,"Routing\tPolicy " << m_name << " destination matched "
@@ -1267,7 +1308,7 @@ bool ENUMPolicy::FindByAliases(RoutingRequest & request, H225_ArrayOf_AliasAddre
 			alias = alias.Mid(1);
 		PINDEX j;
 		for (j = 0; j < alias.GetLength(); ++j)
-			if (!isdigit(alias[j]))
+			if (!isdigit(static_cast<unsigned char>(alias[j])))
 				break;
 
 		if (j >= alias.GetLength()) {
@@ -1295,6 +1336,33 @@ bool ENUMPolicy::FindByAliases(LocationRequest & /* request */, H225_ArrayOf_Ali
 }
 
 
+
+class DestinationRoutes {
+public:
+	DestinationRoutes() { m_endChain = false; m_reject = false; m_aliasesChanged = false;};
+	~DestinationRoutes() { };
+	
+	bool EndPolicyChain() const { return m_endChain; };
+	bool RejectCall() const { return m_reject; };
+	void SetRejectCall(bool reject) { m_reject = reject; m_endChain = true; };
+	unsigned int GetRejectReason() const { return m_rejectReason; };
+	void SetRejectReason(unsigned int reason) { m_rejectReason = reason; };
+	bool ChangeAliases() const { return m_aliasesChanged; };
+	H225_ArrayOf_AliasAddress GetNewAliases() const { return m_newAliases; };
+	void SetNewAliases(const H225_ArrayOf_AliasAddress & aliases) { m_newAliases = aliases; m_aliasesChanged = true; };
+	
+	void AddRoute(const Route & route) { m_routes.push_back(route); m_endChain = true; };
+	
+	std::list<Route> m_routes;
+
+protected:
+	bool m_endChain;
+	bool m_reject;
+	unsigned int m_rejectReason;
+	bool m_aliasesChanged;
+	H225_ArrayOf_AliasAddress m_newAliases;
+};
+
 // a policy to route calls via an SQL database
 class SqlPolicy : public Policy {
 public:
@@ -1315,10 +1383,9 @@ protected:
 		const PString & calledIP,
 		const PString & caller,
 		const PString & callid,
+		const PString & messageType,
 		/* out: */
-		H225_ArrayOf_AliasAddress ** newAliases,	/* DatabaseLookup will allocate, caller must delete */
-		H225_TransportAddress ** newCallSigAdr,		/* DatabaseLookup will allocate, caller must delete */
-		bool * reject);
+		DestinationRoutes & destination);
 
 private:
 	// active ?
@@ -1334,7 +1401,7 @@ private:
 SqlPolicy::SqlPolicy()
 {
 	m_active = false;
-#if HAS_MYSQL || HAS_PGSQL || HAS_FIREBIRD
+#if HAS_DATABASE
 	m_active = true;
 	static const char *sqlsection = "Routing::Sql";
 	m_name = "SqlPolicy";
@@ -1378,7 +1445,7 @@ SqlPolicy::SqlPolicy()
 	}
 #else
 	PTRACE(1, m_name << " not available - no database driver compiled into GnuGk");
-#endif // HAS_MYSQL || HAS_PGSQL || HAS_FIREBIRD
+#endif // HAS_DATABASE
 }
 
 SqlPolicy::~SqlPolicy()
@@ -1399,33 +1466,35 @@ bool SqlPolicy::OnRequest(AdmissionRequest & request)
 		PString calledIP = "";	/* not available for ARQs */
 		PString caller = AsString(arq.m_srcInfo, FALSE);
 		PString callid = AsString(arq.m_callIdentifier.m_guid);
-		H225_ArrayOf_AliasAddress * newAliases = NULL;
-		H225_TransportAddress * newCallSigAdr = NULL;
-		bool reject = false;
+		PString messageType = "ARQ";
+		DestinationRoutes destination;
 
-		DatabaseLookup(	/* in */ source, calledAlias, calledIP, caller, callid,
-						/* out: */ &newAliases, &newCallSigAdr, &reject);
+		DatabaseLookup(	/* in */ source, calledAlias, calledIP, caller, callid, messageType,
+						/* out: */ destination);
 
-		if (reject) {
+		if (destination.RejectCall()) {
 			request.SetFlag(RoutingRequest::e_Reject);
+			request.SetRejectReason(destination.GetRejectReason());
 		}
 
-		if (newAliases) {
+		if (destination.ChangeAliases()) {
 			request.SetFlag(RoutingRequest::e_aliasesChanged);
-			arq.m_destinationInfo = *newAliases;
-			delete newAliases;
+			arq.m_destinationInfo = destination.GetNewAliases();
 		}
 
-		if (newCallSigAdr) {
+		if (!destination.m_routes.empty()) {
 			request.SetFlag(RoutingRequest::e_aliasesChanged);
 			if (!arq.HasOptionalField(H225_AdmissionRequest::e_destCallSignalAddress)) {
 				arq.IncludeOptionalField(H225_AdmissionRequest::e_destCallSignalAddress);
 			}
-			arq.m_destCallSignalAddress = *newCallSigAdr;
-			delete newCallSigAdr;
+			arq.m_destCallSignalAddress = destination.m_routes.front().m_destAddr;
+			while (!destination.m_routes.empty()) {
+				request.AddRoute(destination.m_routes.front());
+				destination.m_routes.pop_front();
+			}
 		}
 
-		if (m_next == NULL || reject)
+		if (m_next == NULL || destination.EndPolicyChain())
 			return true;
 	}
 	return false;
@@ -1447,37 +1516,32 @@ bool SqlPolicy::OnRequest(LocationRequest & request)
 	if (lrq.HasOptionalField(H225_LocationRequest::e_sourceInfo) && (lrq.m_sourceInfo.GetSize() > 0))
 		caller = AsString(lrq.m_sourceInfo[0], FALSE);
 	PString callid = "";	/* not available for LRQs */
+	PString messageType = "LRQ";
+	DestinationRoutes destination;
 
-	H225_ArrayOf_AliasAddress * newAliases = NULL;
-	H225_TransportAddress * newCallSigAdr = NULL;
-	bool reject = false;
+	DatabaseLookup(	/* in */ source, calledAlias, calledIP, caller, callid, messageType,
+					/* out: */ destination);
 
-	DatabaseLookup(	/* in */ source, calledAlias, calledIP, caller, callid,
-					/* out: */ &newAliases, &newCallSigAdr, &reject);
-
-	if (reject) {
+	if (destination.RejectCall()) {
 		request.SetFlag(RoutingRequest::e_Reject);
+		request.SetRejectReason(destination.GetRejectReason());
 	}
 
-	if (newAliases) {
+	if (destination.ChangeAliases()) {
 		request.SetFlag(RoutingRequest::e_aliasesChanged);
-		lrq.m_destinationInfo = *newAliases;
-		delete newAliases;
+		lrq.m_destinationInfo = destination.GetNewAliases();
 	}
 
-	if (newCallSigAdr) {
+	if (!destination.m_routes.empty()) {
 		// 'explicit' policy can't handle LRQs, so we do it directly
 		request.SetFlag(RoutingRequest::e_aliasesChanged);
-		Route route("Sql", *newCallSigAdr);
-		route.m_destEndpoint = RegistrationTable::Instance()->FindBySignalAdr(
-			route.m_destAddr
-			);
-		request.AddRoute(route);
-		delete newCallSigAdr;
-		return true;	// stop processing
+		while (!destination.m_routes.empty()) {
+			request.AddRoute(destination.m_routes.front());
+			destination.m_routes.pop_front();
+		}
 	}
 
-	if (m_next == NULL || reject)
+	if (m_next == NULL || destination.EndPolicyChain())
 		return true;
 
 	return false;
@@ -1498,37 +1562,38 @@ bool SqlPolicy::OnRequest(SetupRequest & request)
 	PString calledIP = localAddr;
 	PString caller = AsString(setup.m_sourceAddress, FALSE);
 	PString callid = AsString(setup.m_callIdentifier.m_guid);
+	PString messageType = "Setup";
+	DestinationRoutes destination;
 
-	H225_ArrayOf_AliasAddress * newAliases = NULL;
-	H225_TransportAddress * newCallSigAdr = NULL;
-	bool reject = false;
+	DatabaseLookup(	/* in */ source, calledAlias, calledIP, caller, callid, messageType,
+					/* out: */ destination);
 
-	DatabaseLookup(	/* in */ source, calledAlias, calledIP, caller, callid,
-					/* out: */ &newAliases, &newCallSigAdr, &reject);
-
-	if (reject) {
+	if (destination.RejectCall()) {
 		request.SetFlag(RoutingRequest::e_Reject);
+		request.SetRejectReason(destination.GetRejectReason());
 	}
 
-	if (newAliases) {
+	if (destination.ChangeAliases()) {
 		request.SetFlag(RoutingRequest::e_aliasesChanged);
 		if (!setup.HasOptionalField(H225_Setup_UUIE::e_destinationAddress)) {
 			setup.IncludeOptionalField(H225_Setup_UUIE::e_destinationAddress);
 		}
-		setup.m_destinationAddress = *newAliases;
-		delete newAliases;
+		setup.m_destinationAddress = destination.GetNewAliases();
 	}
 
-	if (newCallSigAdr) {
+	if (!destination.m_routes.empty()) {
 		request.SetFlag(RoutingRequest::e_aliasesChanged);
 		if (!setup.HasOptionalField(H225_Setup_UUIE::e_destCallSignalAddress)) {
 			setup.IncludeOptionalField(H225_Setup_UUIE::e_destCallSignalAddress);
 		}
-		setup.m_destCallSignalAddress = *newCallSigAdr;
-		delete newCallSigAdr;
+		setup.m_destCallSignalAddress = destination.m_routes.front().m_destAddr;
+		while (!destination.m_routes.empty()) {
+			request.AddRoute(destination.m_routes.front());
+			destination.m_routes.pop_front();
+		}
 	}
 
-	if (m_next == NULL || reject)
+	if (m_next == NULL || destination.EndPolicyChain())
 		return true;
 
 	return false;
@@ -1541,12 +1606,11 @@ void SqlPolicy::DatabaseLookup(
 		const PString & calledIP,
 		const PString & caller,
 		const PString & callid,
+		const PString & messageType,
 		/* out: */
-		H225_ArrayOf_AliasAddress ** newAliases,	/* DatabaseLookup will allocate, caller must delete */
-		H225_TransportAddress ** newCallSigAdr,	/* DatabaseLookup will allocate, caller must delete */
-		bool * reject)
+		DestinationRoutes & destination)
 {
-#if HAS_MYSQL || HAS_PGSQL || HAS_FIREBIRD
+#if HAS_DATABASE
 	GkSQLResult::ResultRow resultRow;
 	std::map<PString, PString> params;
 	params["s"] = source;
@@ -1554,6 +1618,7 @@ void SqlPolicy::DatabaseLookup(
 	params["p"] = calledIP;
 	params["r"] = caller;
 	params["i"] = callid;
+	params["m"] = messageType;
 	GkSQLResult* result = m_sqlConn->ExecuteQuery(m_query, params, m_timeout);
 	if (result == NULL) {
 		PTRACE(2, m_name << ": query failed - timeout or fatal error");
@@ -1578,40 +1643,73 @@ void SqlPolicy::DatabaseLookup(
 		PTRACE(2, m_name << ": query failed - could not fetch the result row");
 	else if (result->GetNumFields() == 1) {
 		PString newDestination = resultRow[0].first;
-		PTRACE(5, m_name << "\tQuery result: " << newDestination);
-		*reject = FALSE;
+		PTRACE(5, m_name << "\tQuery result : " << newDestination);
 		if (newDestination.ToUpper() == "REJECT") {
-			*reject = TRUE;
+			destination.SetRejectCall(true);
 		} else if (IsIPAddress(newDestination)) {
-			*newCallSigAdr = new H225_TransportAddress();
-			PStringArray adr_parts = newDestination.Tokenise(":", FALSE);
-			PString ip = adr_parts[0];
-			WORD port = (WORD)(adr_parts[1].AsInteger());
-			if (port == 0)
-				port = GK_DEF_ENDPOINT_SIGNAL_PORT;
-			**newCallSigAdr = SocketToH225TransportAddr(ip, port);
+			int row = 0;
+			do {
+				PString destinationIp = resultRow[0].first;
+				PStringArray adr_parts = destinationIp.Tokenise(":", FALSE);
+				PString ip = adr_parts[0];
+				WORD port = (WORD)(adr_parts[1].AsInteger());
+				if (port == 0)
+					port = GK_DEF_ENDPOINT_SIGNAL_PORT;
+
+				Route route("Sql", SocketToH225TransportAddr(ip, port));
+				route.m_destEndpoint = RegistrationTable::Instance()->FindBySignalAdr(route.m_destAddr);
+				destination.AddRoute(route);
+
+				row++;
+				if (row < result->GetNumRows()) {
+					result->FetchRow(resultRow);	// fetch next row
+					PTRACE(5, m_name << "\tResult cont'd: " << resultRow[0].first);
+				}
+			} while (row < result->GetNumRows());
 		} else {
-			*newAliases = new H225_ArrayOf_AliasAddress();
-			(*newAliases)->SetSize(1);
-			H323SetAliasAddress(newDestination, (**newAliases)[0]);
+			H225_ArrayOf_AliasAddress newAliases;
+			newAliases.SetSize(1);
+			H323SetAliasAddress(newDestination, newAliases[0]);
+			destination.SetNewAliases(newAliases);
 		}
 	} else if (result->GetNumFields() == 2) {
 		PString newDestinationAlias = resultRow[0].first;
 		PString newDestinationIP = resultRow[1].first;
-		PTRACE(5, m_name << "\tQuery result: " << newDestinationAlias << ", " << newDestinationIP);
-		*newAliases = new H225_ArrayOf_AliasAddress();
-		(*newAliases)->SetSize(1);
-		H323SetAliasAddress(newDestinationAlias, (**newAliases)[0]);
-		*newCallSigAdr = new H225_TransportAddress();
-		PStringArray adr_parts = newDestinationIP.Tokenise(":", FALSE);
-		PString ip = adr_parts[0];
-		WORD port = (WORD)(adr_parts[1].AsInteger());
-		if (port == 0)
-			port = GK_DEF_ENDPOINT_SIGNAL_PORT;
-		**newCallSigAdr = SocketToH225TransportAddr(ip, port);
+		PTRACE(5, m_name << "\tQuery result : " << newDestinationAlias << ", " << newDestinationIP);
+		if (newDestinationAlias.ToUpper() == "REJECT") {
+			destination.SetRejectCall(true);;
+			destination.SetRejectReason(newDestinationIP.AsInteger());
+		} else {
+			H225_ArrayOf_AliasAddress newAliases;
+			newAliases.SetSize(1);
+			H323SetAliasAddress(newDestinationAlias, newAliases[0]);
+			destination.SetNewAliases(newAliases);
+			int row = 0;
+			do {
+				PString destinationAlias = resultRow[0].first;
+				PString destinationIp = resultRow[1].first;
+				PStringArray adr_parts = destinationIp.Tokenise(":", FALSE);
+				PString ip = adr_parts[0];
+				WORD port = (WORD)(adr_parts[1].AsInteger());
+				if (port == 0)
+					port = GK_DEF_ENDPOINT_SIGNAL_PORT;
+
+				Route route("Sql", SocketToH225TransportAddr(ip, port));
+				route.m_destEndpoint = RegistrationTable::Instance()->FindBySignalAdr(route.m_destAddr);
+				route.m_destNumber = destinationAlias;
+				destination.AddRoute(route);
+
+				row++;
+				if (row < result->GetNumRows()) {
+					result->FetchRow(resultRow);	// fetch next row
+					PTRACE(5, m_name << "\tResult cont'd: " << resultRow[0].first << ", " << resultRow[1].first);
+				
+				}
+			} while (row < result->GetNumRows());
+		}
 	}
 	delete result;
-#endif // HAS_MYSQL || HAS_PGSQL || HAS_FIREBIRD
+#endif // HAS_DATABASE
 }
 
 
