@@ -17,7 +17,7 @@
 //////////////////////////////////////////////////////////////////
 
 #ifndef ROUTING_H
-#define ROUTING_H "@(#) $Id: Routing.h,v 1.30 2008/01/16 12:14:43 willamowius Exp $"
+#define ROUTING_H "@(#) $Id: Routing.h,v 1.32 2008/09/15 08:42:09 zvision Exp $"
 
 #include <map>
 #include <list>
@@ -78,7 +78,8 @@ struct Route {
 	int m_proxyMode; /// per-route proxy mode flag
 	unsigned m_flags; /// additional route specific flags
 	unsigned char m_rerouteCauses[16]; /// bit flags to trigger rerouting on particular Q931 causes
-	PString m_destNumber;
+	PString m_destNumber; /// rewritten number (corresponds to Toolkit::RewriteE164)
+	PString m_destOutNumber; /// number actually sent to the called party  (corresponds to Toolkit::GWRewriteE164)
 };
 
 class RoutingRequest {
@@ -188,7 +189,7 @@ public:
 protected:
 	// new virtual function
 	// if return false, the policy is disable
-	virtual bool IsActive() { return true; }
+	virtual bool IsActive() const { return true; }
 
 	// methods to handle the request
 	// return true:  fate of the request is determined (confirm or reject)
