@@ -27,7 +27,8 @@
 #include "h323util.h"
 #include "GkStatus.h"
 #include "gkconfig.h"
-#if HAS_MYSQL || HAS_PGSQL || HAS_FIREBIRD
+#include "config.h"
+#if HAS_DATABASE
 #include "gksql.h"
 #endif
 #include "clirw.h"
@@ -262,13 +263,29 @@ void Toolkit::RouteTable::InitTable()
 	if (!CreateTable())
 		return;
 
-	// Set default IP according to route table
-	PIPSocket::Address defGW;
-	PIPSocket::GetGatewayAddress(defGW);
-	if (defGW.AsString() == "0.0.0.0")
-		PIPSocket::GetNetworkInterface(defAddr);
-	else
-		defAddr = GetLocalAddress(defGW);
+	// Get all the interface addresses available
+	std::vector<PIPSocket::Address> home;
+	PString foo = Toolkit::Instance()->GetGKHome(home);
+	if (!home.empty() && home.size() > 1) {
+		// Set default IP to Bind IP, if given.
+		PString bind = GkConfig()->GetString("Bind", "");
+		if (!bind) {
+			defAddr = bind;
+		} else {
+			defAddr = home[0];
+		}
+	}
+
+	// If we do not already have a valid entry, try and retrieve the default interface
+	if (defAddr.IsLoopback() || !defAddr.IsValid()) {
+		// Set default IP according to route table
+		PIPSocket::Address defGW;
+		PIPSocket::GetGatewayAddress(defGW);
+		if (defGW.AsString() == "0.0.0.0")
+			PIPSocket::GetNetworkInterface(defAddr);
+		else
+			defAddr = GetLocalAddress(defGW);
+    }
 
 #if PTRACING
 	for (RouteEntry *entry = rtable_begin; entry != rtable_end; ++entry)
@@ -468,7 +485,7 @@ void Toolkit::RewriteData::AddSection(PConfig *config, const PString & section)
 		for (PINDEX i = 0; i < n_size; ++i) {
 			PString key = cfgs.GetKeyAt(i);
 			PCaselessString first = PCaselessString(key[0]);				
-			if (!key && (isdigit(key[0]) || (first.FindOneOf("!.%*#ABCDEFGHIGKLMNOPQRSTUVWXYZ") != P_MAX_INDEX)))			
+			if (!key && (isdigit(static_cast<unsigned char>(key[0])) || (first.FindOneOf("!.%*#ABCDEFGHIGKLMNOPQRSTUVWXYZ") != P_MAX_INDEX)))			
 				rules[key] = cfgs.GetDataAt(i);
 		}
 		// now the rules are ascendantly sorted by the keys
@@ -544,7 +561,7 @@ bool Toolkit::RewriteTool::RewritePString(PString & s) const
 			 // sending the LRQ. 
 			 PINDEX j;
  		     for (j = 0; j < num.GetLength(); ++j)
-			       if (!isdigit(num[j]))
+			       if (!isdigit(static_cast<unsigned char>(num[j])))
 			   	         break;
 
 			 if (j >= num.GetLength()) { // is numeric
@@ -907,8 +924,12 @@ void Toolkit::CreateConfig()
 			delete m_Config;
 			m_Config = testConfig;
 		} else {
-			PTRACE(0, "CONFIG\tFailed to read valid config - keeping old config");
-			GkStatus::Instance()->SignalStatus("Failed to read valid config - keeping old config\r\n", MIN_STATUS_TRACE_LEVEL);
+			if (m_Config) {	// reload
+				PTRACE(0, "CONFIG\tFailed to read valid config - keeping old config");
+				GkStatus::Instance()->SignalStatus("Failed to read valid config - keeping old config\r\n", MIN_STATUS_TRACE_LEVEL);
+			} else {	// startup
+				m_Config = testConfig;	// warning will be printed a bit later			
+			}
 		}
 	} else { // Oops! Create temporary config file failed, use the original one
 		PTRACE(0, "CONFIG\tCould not create/link config to a temporary file " << m_tmpconfig);
@@ -932,7 +953,7 @@ void Toolkit::CreateConfig()
 
 void Toolkit::ReloadSQLConfig()
 {
-#if HAS_MYSQL || HAS_PGSQL || HAS_FIREBIRD
+#if HAS_DATABASE
 	if (m_Config->GetSections().GetStringsIndex("SQLConfig") == P_MAX_INDEX)
 		return;
 
@@ -1260,15 +1281,22 @@ void Toolkit::ReloadSQLConfig()
 	delete sqlConn;
 	sqlConn = NULL;
 	PTRACE(3, "SQLCONF\tSQL config connection closed");
-#endif // HAS_MYSQL || HAS_PGSQL || HAS_FIREBIRD
+#endif // HAS_DATABASE
+}
+
+void Toolkit::PrepareReloadConfig()
+{
+	if (!m_ConfigDirty)
+		CreateConfig();
+	else // the config have been changed via status port, use it directly
+		m_ConfigDirty = false;
 }
 
 PConfig* Toolkit::ReloadConfig()
 {
-	if (!m_ConfigDirty)
-		CreateConfig();
-	else // the config has been changed via status port, use it directly
-		m_ConfigDirty = false;
+	// make a new symlink if needed
+	PrepareReloadConfig();
+	// read the toolkit config values
 
 	m_GKName = Config()->GetString("Name", "OpenH323GK");
 	
@@ -1281,6 +1309,12 @@ PConfig* Toolkit::ReloadConfig()
 		m_encKeyPaddingByte = m_encryptAllPasswords ? 0 : -1;
 		
 	ReloadSQLConfig();
+
+	// TODO/BUG: always call SetGKHome() on reload, even if we don't have a Home= setting
+	// otherwise we won't detect new IPs on the machine
+	PString GKHome(m_Config->GetString("Home", ""));
+	if (m_GKHome.empty() || !GKHome)
+		SetGKHome(GKHome.Tokenise(",:;", false));
 	
 	m_RouteTable.InitTable();
 	m_VirtualRouteTable.InitTable();
@@ -1291,13 +1325,10 @@ PConfig* Toolkit::ReloadConfig()
 #ifdef h323v6
 	m_AssignedGKs.LoadConfig(m_Config);
 #endif
-#if HAS_MYSQL || HAS_PGSQL || HAS_FIREBIRD
+#if HAS_DATABASE
 	m_qosMonitor.LoadConfig(m_Config);
 #endif
-	PString GKHome(m_Config->GetString("Home", ""));
-	if (m_GKHome.empty() || !GKHome)
-		SetGKHome(GKHome.Tokenise(",:;", false));
-	
+
 	m_timestampFormatStr = Config()->GetString("TimestampFormat", "Cisco");
 
 	delete m_cliRewrite;
@@ -1308,6 +1339,9 @@ PConfig* Toolkit::ReloadConfig()
 	LoadCauseMap(m_Config);
 
 	LoadReasonMap(m_Config);
+
+	ParseTranslationMap(m_receivedCauseMap, Config()->GetString(RoutedSec, "TranslateReceivedQ931Cause", ""));
+	ParseTranslationMap(m_sentCauseMap, Config()->GetString(RoutedSec, "TranslateSentQ931Cause", ""));
 	
 	return m_Config;
 }
@@ -1333,7 +1367,7 @@ void Toolkit::LoadCauseMap(
 				unsigned cmin = causeRange[0].AsUnsigned() & 0x7f;
 				unsigned cmax = causeRange[1].AsUnsigned() & 0x7f;
 				for (; cmin <= cmax; ++cmin)
-					m_causeMap[cmin >> 3] |= (1UL << (cmin & 7));
+					m_causeMap[(cmin >> 3) & 0x0f] |= (1UL << (cmin & 7));
 			}
 		}
 }
@@ -1351,9 +1385,7 @@ void Toolkit::LoadReasonMap(
 	m_H225ReasonToQ931Cause.assign(&DefaultH225ReasonToQ931Cause[0], &DefaultH225ReasonToQ931Cause[22]);
 
 	for(int reason = 0; reason < H225_ReleaseCompleteReason::e_tunnelledSignallingRejected; reason++) {
-		PString str_reason;
-		str_reason.sprintf("%d", reason);
-		PString cause = cfg->GetString("H225toQ931", str_reason, "");
+		PString cause = cfg->GetString("H225toQ931", PString(reason), "");
 		if (!cause.IsEmpty()) {
 			m_H225ReasonToQ931Cause[reason] = cause.AsInteger();
 		}
@@ -1376,7 +1408,7 @@ bool Toolkit::MatchRegex(const PString &str, const PString &regexStr)
 	return TRUE;
 }
 
-#if HAS_MYSQL || HAS_PGSQL || HAS_FIREBIRD
+#if HAS_DATABASE
 Toolkit::AssignedAliases::AssignedAliases()
   : m_sqlactive(false), m_sqlConn(NULL)
 {
@@ -1479,13 +1511,13 @@ bool Toolkit::AssignedAliases::DatabaseLookup(
 
    return success;
 }
-#endif   // HAS_MYSQL || HAS_PGSQL || HAS_FIREBIRD
+#endif   // HAS_DATABASE
 
 void Toolkit::AssignedAliases::LoadConfig(PConfig * m_config)
 {
 	gkAssignedAliases.clear();
 
-#if HAS_MYSQL || HAS_PGSQL || HAS_FIREBIRD
+#if HAS_DATABASE
   if (LoadSQL(m_config)) {
 	  m_sqlactive = true;
   } else 
@@ -1603,7 +1635,7 @@ bool Toolkit::AssignedAliases::QueryH350Directory(const PString & alias, PString
 
 bool Toolkit::AssignedAliases::QueryAssignedAliases(const PString & alias, PStringArray & aliases)
 {
-#if HAS_MYSQL || HAS_PGSQL || HAS_FIREBIRD
+#if HAS_DATABASE
 	if (DatabaseLookup(alias, aliases))
 		return true;
 #endif
@@ -1671,7 +1703,7 @@ bool Toolkit::AssignedAliases::GetAliases(const H225_ArrayOf_AliasAddress & alia
 
 ////////////////////////////////////////////////////////////////////////
 #ifdef h323v6
-#if HAS_MYSQL || HAS_PGSQL || HAS_FIREBIRD
+#if HAS_DATABASE
 Toolkit::AssignedGatekeepers::AssignedGatekeepers()
   : m_sqlactive(false), m_sqlConn(NULL)
 {
@@ -1776,13 +1808,13 @@ bool Toolkit::AssignedGatekeepers::DatabaseLookup(
 
    return success;
 }
-#endif   // HAS_MYSQL || HAS_PGSQL || HAS_FIREBIRD
+#endif   // HAS_DATABASE
 
 void Toolkit::AssignedGatekeepers::LoadConfig(PConfig * m_config)
 {
 	assignedGKList.clear();
 
-#if HAS_MYSQL || HAS_PGSQL || HAS_FIREBIRD
+#if HAS_DATABASE
   if (LoadSQL(m_config)) {
 	  m_sqlactive = true;
   } else 
@@ -1854,7 +1886,7 @@ bool Toolkit::AssignedGatekeepers::QueryH350Directory(const PString & alias,cons
 
 bool Toolkit::AssignedGatekeepers::QueryAssignedGK(const PString & alias,const PIPSocket::Address & ip, PStringArray & addresses)
 {
-#if HAS_MYSQL || HAS_PGSQL || HAS_FIREBIRD
+#if HAS_DATABASE
 	if (DatabaseLookup(alias,ip,addresses))
 		return true;
 #endif
@@ -1951,7 +1983,7 @@ bool Toolkit::AssignedGatekeepers::GetAssignedGK(const PString & alias,const PIP
 
 /////////////////////////////////////////////////////////////////////////////////////////
 
-#if HAS_MYSQL || HAS_PGSQL || HAS_FIREBIRD
+#if HAS_DATABASE
 Toolkit::QoSMonitor::QoSMonitor()
   : m_sqlactive(false), m_sqlConn(NULL)
 {
@@ -2045,15 +2077,32 @@ bool Toolkit::RewriteE164(H225_AliasAddress &alias)
 {
 	if ((alias.GetTag() != H225_AliasAddress::e_dialedDigits) &&
          (alias.GetTag() != H225_AliasAddress::e_h323_ID) &&
-		 (alias.GetTag() != H225_AliasAddress::e_url_ID))
-		return FALSE;
-
+		 (alias.GetTag() != H225_AliasAddress::e_url_ID)) {
+		if (alias.GetTag() != H225_AliasAddress::e_partyNumber)
+			return false;
+		H225_PartyNumber &partyNumber = alias;
+		if (partyNumber.GetTag() != H225_PartyNumber::e_e164Number && partyNumber.GetTag() != H225_PartyNumber::e_privateNumber)
+			return false;
+	}
+	
 	PString E164 = ::AsString(alias, FALSE);
 
 	bool changed = RewritePString(E164);
-	if (changed)
-		H323SetAliasAddress(E164, alias);
-
+	if (changed) {
+		if (alias.GetTag() == H225_AliasAddress::e_dialedDigits)
+			H323SetAliasAddress(E164, alias, alias.GetTag());
+		else {
+			H225_PartyNumber &partyNumber = alias;
+			if (partyNumber.GetTag() == H225_PartyNumber::e_e164Number) {
+				H225_PublicPartyNumber &number = partyNumber;
+				number.m_publicNumberDigits = E164;
+			} else if (partyNumber.GetTag() == H225_PartyNumber::e_privateNumber) {
+				H225_PrivatePartyNumber &number = partyNumber;
+				number.m_privateNumberDigits = E164;
+			}
+		}
+	}
+	
 	return changed;
 }
 
@@ -2071,14 +2120,29 @@ bool Toolkit::GWRewriteE164(PString gw, bool direction, H225_AliasAddress &alias
 	bool changed;
 
 	if (alias.GetTag() != H225_AliasAddress::e_dialedDigits) {
-		return false;
+		if (alias.GetTag() != H225_AliasAddress::e_partyNumber)
+			return false;
+		H225_PartyNumber &partyNumber = alias;
+		if (partyNumber.GetTag() != H225_PartyNumber::e_e164Number && partyNumber.GetTag() != H225_PartyNumber::e_privateNumber)
+			return false;
 	}
 
 	E164 = ::AsString(alias, FALSE);
 	changed = GWRewritePString(gw,direction,E164);
 
 	if (changed) {
-		H323SetAliasAddress(E164, alias);
+		if (alias.GetTag() == H225_AliasAddress::e_dialedDigits)
+			H323SetAliasAddress(E164, alias, alias.GetTag());
+		else {
+			H225_PartyNumber &partyNumber = alias;
+			if (partyNumber.GetTag() == H225_PartyNumber::e_e164Number) {
+				H225_PublicPartyNumber &number = partyNumber;
+				number.m_publicNumberDigits = E164;
+			} else if (partyNumber.GetTag() == H225_PartyNumber::e_privateNumber) {
+				H225_PrivatePartyNumber &number = partyNumber;
+				number.m_privateNumberDigits = E164;
+			}
+		}
 	}
 
 	return changed;
@@ -2359,28 +2423,32 @@ PString Toolkit::ReadPassword(
 		return PString();
 
 	int paddingByte = m_encKeyPaddingByte;
-	if (cfg->HasKey(cfgSection, paddingByteConfigKey))
+	if (cfg->HasKey(cfgSection, paddingByteConfigKey)) {
 		paddingByte = cfg->GetInteger(cfgSection, paddingByteConfigKey, 0);
+	}
 
-	if (paddingByte == -1)
-		if (forceEncrypted || m_encryptAllPasswords)
+	if (paddingByte == -1) {
+		if (forceEncrypted || m_encryptAllPasswords) {
 			paddingByte = 0;
-		else
+		} else {
 			return cfg->GetString(cfgSection, cfgKey, "");
+		}
+	}
 
 	PTEACypher::Key encKey;
 	memset(&encKey, paddingByte, sizeof(encKey));
 
 	const size_t keyLen = cfgKey.GetLength();
-	if (keyLen > 0)
+	if (keyLen > 0) {
 		memcpy(&encKey, (const char*)cfgKey, min(keyLen, sizeof(encKey)));
+	}
 
 	PTEACypher cypher(encKey);
 	PString s;
-	if (!cypher.Decode(cfg->GetString(cfgSection, cfgKey, ""), s))
+	if (!cypher.Decode(cfg->GetString(cfgSection, cfgKey, ""), s)) {
 		PTRACE(1, "GK\tFailed to decode config password for [" << cfgSection
-			<< "] => " << cfgKey
-			);
+			<< "] => " << cfgKey);
+	}
 	return s;
 }
 
@@ -2418,6 +2486,38 @@ unsigned Toolkit::MapH225ReasonToQ931Cause(
 		return m_H225ReasonToQ931Cause[reason];
 }
 
+void Toolkit::ParseTranslationMap(std::map<unsigned, unsigned> & cause_map, const PString & ini) const
+{
+	cause_map.clear();
+	PStringArray pairs(ini.Tokenise(",", false));
+	for (PINDEX i = 0; i < pairs.GetSize(); ++i) {
+		PStringArray causes(pairs[i].Tokenise(":=", false));
+		if (causes.GetSize() == 2) {
+			cause_map.insert(pair<unsigned, unsigned>(causes[0].AsInteger(), causes[1].AsInteger()));
+		} else {
+			PTRACE(1, "Syntax error in cause mapping: " << causes[i]);
+		}
+	}
+}
+
+unsigned Toolkit::TranslateReceivedCause(unsigned cause) const
+{
+	std::map<unsigned, unsigned>::const_iterator i = m_receivedCauseMap.find(cause);
+	if (i != m_receivedCauseMap.end())
+		return i->second;
+	else
+		return cause;
+}
+
+unsigned Toolkit::TranslateSentCause(unsigned cause) const
+{
+	std::map<unsigned, unsigned>::const_iterator i = m_sentCauseMap.find(cause);
+	if (i != m_sentCauseMap.end())
+		return i->second;
+	else
+		return cause;
+}
+
 #ifdef OpenH323Factory
 PStringList Toolkit::GetAuthenticatorList()
 {
