@@ -10,12 +10,13 @@
 //////////////////////////////////////////////////////////////////
 
 #ifndef TOOLKIT_H
-#define TOOLKIT_H "@(#) $Id: Toolkit.h,v 1.55 2007/12/19 01:10:53 willamowius Exp $"
+#define TOOLKIT_H "@(#) $Id: Toolkit.h,v 1.59 2008/11/08 01:35:21 willamowius Exp $"
 
 #include <vector>
+#include <map>
 #include <ptlib/sockets.h>
 #include "singleton.h"
-#include "pwlib_compat.h"
+#include "config.h"
 
 class H225_AliasAddress;
 class H225_ArrayOf_AliasAddress;
@@ -91,7 +92,7 @@ bool operator<<(const PIPSocket::Address &addr, const NetworkAddress &net);
 #ifdef H323_H350
 class H350_Session;
 #endif
-#if HAS_MYSQL || HAS_PGSQL || HAS_FIREBIRD
+#if HAS_DATABASE
 class GkSQLConnection;
 #endif
 class GkTimerManager;
@@ -206,7 +207,7 @@ class Toolkit : public Singleton<Toolkit>
 
 	class AssignedAliases {
 	 public:
-#if HAS_MYSQL || HAS_PGSQL || HAS_FIREBIRD
+#if HAS_DATABASE
 	    AssignedAliases();
 		~AssignedAliases();
 
@@ -223,7 +224,7 @@ class Toolkit : public Singleton<Toolkit>
 	 protected:
 		 std::vector< std::pair<PString, PString> > gkAssignedAliases;
 
-#if HAS_MYSQL || HAS_PGSQL || HAS_FIREBIRD
+#if HAS_DATABASE
      private:
 	   bool m_sqlactive;
 	   // connection to the SQL database
@@ -240,7 +241,7 @@ class Toolkit : public Singleton<Toolkit>
 #ifdef h323v6
 	class AssignedGatekeepers {
 	 public:
-#if HAS_MYSQL || HAS_PGSQL || HAS_FIREBIRD
+#if HAS_DATABASE
 	    AssignedGatekeepers();
 		~AssignedGatekeepers();
 
@@ -257,7 +258,7 @@ class Toolkit : public Singleton<Toolkit>
 	 protected:
 		 std::vector< std::pair<PString, PString> > assignedGKList;
 
-#if HAS_MYSQL || HAS_PGSQL || HAS_FIREBIRD
+#if HAS_DATABASE
      private:
 	   bool m_sqlactive;
 	   // connection to the SQL database
@@ -273,7 +274,7 @@ class Toolkit : public Singleton<Toolkit>
 	AssignedGatekeepers AssignedGKs() { return m_AssignedGKs; }
 #endif
 
-#if HAS_MYSQL || HAS_PGSQL || HAS_FIREBIRD
+#if HAS_DATABASE
 	class QoSMonitor {
 	  public:
         QoSMonitor();
@@ -382,6 +383,9 @@ class Toolkit : public Singleton<Toolkit>
 	 */
 	void SetConfig(int act, const PString & sec, const PString & key = PString::Empty(), const PString & value = PString::Empty());
 
+	// creates a new symlink so we can re-read the config file
+	void PrepareReloadConfig();
+
 	PConfig* ReloadConfig();
 
 	/// reads name of the running instance from config
@@ -509,23 +513,25 @@ class Toolkit : public Singleton<Toolkit>
 		const PIPSocket::Address &destAddr /// callee's IP
 		) const;
 
-	void SetRerouteCauses(
-		unsigned char *causeMap
-		);
+	void SetRerouteCauses(unsigned char *causeMap);
 
 	/** Map H225_ReleaseCompleteReason code to Q.931 cause value.
 	
 	@return
 	The corresponding Q.931 cause value or 0, if there is no direct mapping.
 	*/	
-	unsigned MapH225ReasonToQ931Cause(
-		int reason
-		);
+	unsigned MapH225ReasonToQ931Cause(int reason);
+
+	void ParseTranslationMap(std::map<unsigned, unsigned> & cause_map, const PString & ini) const;
+	
+	/// global translation of cause codes (called from endpoint specific method)
+	unsigned TranslateReceivedCause(unsigned cause) const;
+	unsigned TranslateSentCause(unsigned cause) const;
 
 #ifdef OpenH323Factory
 	PStringList GetAuthenticatorList();
 #endif
-		
+
 protected:
 	void CreateConfig();
 	void ReloadSQLConfig();
@@ -545,7 +551,7 @@ protected:
 #ifdef h323v6
 	AssignedGatekeepers m_AssignedGKs;
 #endif
-#if HAS_MYSQL || HAS_PGSQL || HAS_FIREBIRD
+#if HAS_DATABASE
 	QoSMonitor m_qosMonitor;
 #endif
 	RouteTable m_RouteTable;
@@ -577,6 +583,9 @@ private:
 	unsigned char m_causeMap[16];
 	/// map H.225 reason to Q.931 cause code
 	vector<unsigned> m_H225ReasonToQ931Cause;
+	/// global cause code translation
+	std::map<unsigned, unsigned> m_receivedCauseMap;
+	std::map<unsigned, unsigned> m_sentCauseMap;
 };
 
 
