@@ -11,10 +11,11 @@
  * with the OpenH323 library.
  */
 #ifndef CAPCTRL_H
-#define CAPCTRL_H "#(@) $Id: capctrl.h,v 1.3 2007/10/19 10:01:40 zvision Exp $"
+#define CAPCTRL_H "#(@) $Id: capctrl.h,v 1.4.2.1 2009/01/06 09:22:13 willamowius Exp $"
 
 #include <string>
 #include <vector>
+#include <set>
 #include "Toolkit.h"
 
 class CallRec;
@@ -33,9 +34,8 @@ public:
 		bool operator==(const InboundCallVolume &) const;
 		
 		std::string m_prefix; /// destination prefix to match (regex)
-		int m_maxVolume; /// maximum allowed call volume
-		int m_currentVolume; /// current call volume
-		std::list<PINDEX> m_calls; /// active calls
+		unsigned m_maxVolume; /// maximum allowed call volume
+		std::set<PINDEX> m_calls; /// active calls
 	};
 
 	struct InboundIPCallVolume : public InboundCallVolume {
@@ -55,6 +55,7 @@ public:
 	};
 
 	struct InboundCLICallVolume : public InboundCallVolume {
+		InboundCLICallVolume() {}	//default c'tor needed for VS 2008 SP1
 		InboundCLICallVolume(const std::string &cli) : m_sourceCLI(cli) {}
 		
 		bool operator==(const InboundCLICallVolume &) const;
