@@ -3,7 +3,7 @@
  *
  * Module for CLI/ANI manipulation.
  *
- * $Id: clirw.cxx,v 1.16 2007/11/25 23:23:39 willamowius Exp $
+ * $Id: clirw.cxx,v 1.18 2008/05/20 18:13:51 willamowius Exp $
  *
  * Copyright (c) 2005, Michal Zygmuntowicz
  *
@@ -81,6 +81,10 @@ PString CLIRewrite::RewriteRule::AsString() const
 		s += " *= ";
 	else if (m_rewriteType == NumberToNumber)
 		s += " ~= ";
+	else if (m_rewriteType == PrefixToH323Id)
+		s += " ^= ";
+	else if (m_rewriteType == NumberToH323Id)
+		s += " /= ";
 	else
 		s += " ?unknown rewrite rule type? ";
 
@@ -164,9 +168,11 @@ CLIRewrite::CLIRewrite()
 		PString key = kv.GetKeyAt(i);
 
 		unsigned j = 0;
-		while (ReservedKeys[j] != NULL)
-			if(key == ReservedKeys[j++])
+		while (ReservedKeys[j] != NULL) {
+			if (key == ReservedKeys[j++]) {
 				break;
+			}
+		}
 		if (ReservedKeys[j] != NULL)
 			continue;
 
@@ -318,11 +324,13 @@ CLIRewrite::CLIRewrite()
 				PTRACE(1, "CLIRW\tInvalid CLI rewrite rule syntax: " << key << '=' 
 					<< kv.GetDataAt(i)
 					);
-				if (newsiprule)
-					if (inbound)
+				if (newsiprule) {
+					if (inbound) {
 						m_inboundRules.erase(siprule);
-					else
+					} else {
 						diprule->second.erase(siprule);
+					}
+				}
 				if (newdiprule)
 					m_outboundRules.erase(diprule);
 				continue;
@@ -346,25 +354,34 @@ CLIRewrite::CLIRewrite()
 			// extract prefix to be matched
 			PINDEX keyChars = sepIndex - keyIndex;
 			int rewriteType = RewriteRule::PrefixToNumber;
-			if (sepIndex > keyIndex)
+			if (sepIndex > keyIndex) {
 				if (data[sepIndex-1] == '*') {
 					rewriteType = RewriteRule::PrefixToPrefix;
 					--keyChars;
 				} else if (data[sepIndex-1] == '~') {
 					rewriteType = RewriteRule::NumberToNumber;
 					--keyChars;
+				} else if (data[sepIndex-1] == '^') {
+					rewriteType = RewriteRule::PrefixToH323Id;
+					--keyChars;
+				} else if (data[sepIndex-1] == '/') {
+					rewriteType = RewriteRule::NumberToH323Id;
+					--keyChars;
 				}
+			}
 					
 			if (rewriteType == RewriteRule::PrefixToPrefix && matchType != RewriteRule::MatchCallerNumber) {
 				PTRACE(1, "CLIRW\tInvalid CLI rewrite rule syntax - cannot perform "
 					"*= rewrite on non 'cli:' rules: " << key << '=' 
 					<< kv.GetDataAt(i)
 					);
-				if (newsiprule)
-					if (inbound)
+				if (newsiprule) {
+					if (inbound) {
 						m_inboundRules.erase(siprule);
-					else
+					} else {
 						diprule->second.erase(siprule);
+					}
+				}
 				if (newdiprule)
 					m_outboundRules.erase(diprule);
 				continue;
@@ -741,8 +758,11 @@ void CLIRewrite::Rewrite(
 	if (presentation != (unsigned)-1 && screening == (unsigned)-1)
 		screening = 0;
 
-	msg.GetQ931().SetCallingPartyNumber(newcli, plan, type, presentation, screening);
-	msg.SetChanged();
+	if (rule->m_rewriteType != RewriteRule::PrefixToH323Id
+			&& rule->m_rewriteType != RewriteRule::NumberToH323Id) {
+		msg.GetQ931().SetCallingPartyNumber(newcli, plan, type, presentation, screening);
+		msg.SetChanged();
+	}
 	if (m_processSourceAddress && msg.GetUUIEBody().HasOptionalField(H225_Setup_UUIE::e_sourceAddress)) {
 		H225_ArrayOf_AliasAddress &sourceAddress = msg.GetUUIEBody().m_sourceAddress;
 		PINDEX aliasIndex = 0;
@@ -750,8 +770,15 @@ void CLIRewrite::Rewrite(
 			sourceAddress.SetSize(1);
 		else {
 			while (aliasIndex < sourceAddress.GetSize())
-				if (sourceAddress[aliasIndex].GetTag() == H225_AliasAddress::e_h323_ID
-						|| sourceAddress[aliasIndex].GetTag() == H225_AliasAddress::e_url_ID
+				if (sourceAddress[aliasIndex].GetTag() == H225_AliasAddress::e_h323_ID) {
+					if (rule->m_rewriteType == RewriteRule::PrefixToH323Id
+							|| rule->m_rewriteType == RewriteRule::NumberToH323Id) {
+						sourceAddress.RemoveAt(aliasIndex);
+					} else {
+						aliasIndex++;
+						continue;
+					}
+				} else if (sourceAddress[aliasIndex].GetTag() == H225_AliasAddress::e_url_ID
 						|| sourceAddress[aliasIndex].GetTag() == H225_AliasAddress::e_email_ID) {
 					aliasIndex++;
 					continue;
@@ -779,11 +806,28 @@ void CLIRewrite::Rewrite(
 			if (isTerminal && !newcli) {
 				msg.GetUUIEBody().IncludeOptionalField(H225_Setup_UUIE::e_screeningIndicator);
 				msg.GetUUIEBody().m_screeningIndicator.SetValue(H225_ScreeningIndicator::e_networkProvided);
-				H323SetAliasAddress(newcli, sourceAddress[sourceAddress.GetSize() - 1]);
+				if (rule->m_rewriteType == RewriteRule::PrefixToH323Id
+						|| rule->m_rewriteType == RewriteRule::NumberToH323Id)
+					H323SetAliasAddress(newcli, sourceAddress[sourceAddress.GetSize() - 1], H225_AliasAddress::e_h323_ID);
+				else
+					H323SetAliasAddress(newcli, sourceAddress[sourceAddress.GetSize() - 1]);
 			} else
 				msg.GetUUIEBody().RemoveOptionalField(H225_Setup_UUIE::e_sourceAddress);
-		} else
-			H323SetAliasAddress(newcli, sourceAddress[sourceAddress.GetSize() - 1]);
+		} else {
+			if (rule->m_rewriteType == RewriteRule::PrefixToH323Id
+					|| rule->m_rewriteType == RewriteRule::NumberToH323Id)
+				H323SetAliasAddress(newcli, sourceAddress[sourceAddress.GetSize() - 1], H225_AliasAddress::e_h323_ID);
+			else
+				H323SetAliasAddress(newcli, sourceAddress[sourceAddress.GetSize() - 1]);
+		}
+		msg.SetUUIEChanged();
+	}
+	
+	if (!msg.GetUUIEBody().HasOptionalField(H225_Setup_UUIE::e_sourceAddress)
+			&& (rule->m_rewriteType == RewriteRule::PrefixToH323Id || rule->m_rewriteType == RewriteRule::NumberToH323Id)) {
+		msg.GetUUIEBody().IncludeOptionalField(H225_Setup_UUIE::e_sourceAddress);
+		msg.GetUUIEBody().m_sourceAddress.SetSize(1);
+		H323SetAliasAddress(newcli, msg.GetUUIEBody().m_sourceAddress[0], H225_AliasAddress::e_h323_ID);
 		msg.SetUUIEChanged();
 	}
 }
