@@ -12,7 +12,7 @@
 //////////////////////////////////////////////////////////////////
 
 #ifndef GKAUTH_H
-#define GKAUTH_H "@(#) $Id: gkauth.h,v 1.39 2008/03/07 15:46:55 zvision Exp $"
+#define GKAUTH_H "@(#) $Id: gkauth.h,v 1.40 2008/05/08 08:22:40 zvision Exp $"
 
 #include <map>
 #include <list>
@@ -339,6 +339,15 @@ public:
 		SetupAuthData& authData
 		);
 
+	/** Get human readable information about current module state
+	    that can be displayed on the status port interface.
+		
+		@return
+		A string (may contain multiple lines) with module information.
+		Each line (including the last one) has to be ended with \r\n.
+	*/
+	virtual PString GetInfo();
+
 protected:
 	/** @return
 	    Default authentication status, if not determined by Check... method.
@@ -968,6 +977,24 @@ public:
 		SetupAuthData& authData
 		);
 
+	/** Get a module information string for the selected module.
+	
+	    @return
+		The module information string for status port diplay.
+	*/
+	PString GetInfo(
+		const PString &moduleName /// module to retrieve information for
+		) {
+		ReadLock lock(m_reloadMutex);
+		std::list<GkAuthenticator*>::const_iterator i = m_authenticators.begin();
+		while (i != m_authenticators.end()) {
+			GkAuthenticator* auth = *i++;
+			if (auth->GetName() == moduleName)
+				return auth->GetInfo();
+		}
+		return moduleName + " module not found\r\n";
+	}
+
 private:
 	GkAuthenticatorList(const GkAuthenticatorList&);
 	GkAuthenticatorList& operator=(const GkAuthenticatorList&);
