@@ -1,7 +1,7 @@
 /*
  * gkconfig.cxx
  *
- * @(#) $Id: gkconfig.cxx,v 1.6 2007/11/27 12:57:27 willamowius Exp $
+ * @(#) $Id: gkconfig.cxx,v 1.7 2008/05/23 10:47:01 willamowius Exp $
  *
  * Custom PConfig implementation that allows chaining together more
  * than one config source (file) and access them through a single config instance.
@@ -14,6 +14,9 @@
  * with the OpenH323 library.
  *
  * $Log: gkconfig.cxx,v $
+ * Revision 1.7  2008/05/23 10:47:01  willamowius
+ * PBoolean patches, H.460 Linux compile fixes
+ *
  * Revision 1.6  2007/11/27 12:57:27  willamowius
  * preparation for PBoolean change in PWLib
  *
@@ -55,9 +58,15 @@ GatekeeperConfig::~GatekeeperConfig()
 	delete m_chainedConfig;
 }
 
+#ifdef hasPConfigArray
+PStringArray GatekeeperConfig::GetSections() const
+{
+	PStringArray list = PConfig::GetSections();
+#else
 PStringList GatekeeperConfig::GetSections() const
 {
 	PStringList list = PConfig::GetSections();
+#endif
 
 	if (m_chainedConfig != NULL) {
 		PStringList chainedList = m_chainedConfig->GetSections();
@@ -69,9 +78,15 @@ PStringList GatekeeperConfig::GetSections() const
 	return list;
 }
 
+#ifdef hasPConfigArray
+PStringArray GatekeeperConfig::GetKeys(const PString & theSection) const
+{
+	PStringArray list = PConfig::GetKeys(theSection);
+#else
 PStringList GatekeeperConfig::GetKeys(const PString & theSection) const
 {
 	PStringList list = PConfig::GetKeys(theSection);
+#endif
 
 	if (m_chainedConfig != NULL) {
 		PStringList chainedList = m_chainedConfig->GetKeys(theSection);
