@@ -12,6 +12,12 @@
  * with the OpenH323 library.
  *
  * $Log: gkconfig.h,v $
+ * Revision 1.8  2008/05/23 10:47:01  willamowius
+ * PBoolean patches, H.460 Linux compile fixes
+ *
+ * Revision 1.7  2008/04/03 10:24:20  willamowius
+ * new header config.h for configuration details
+ *
  * Revision 1.6  2007/11/27 12:57:27  willamowius
  * preparation for PBoolean change in PWLib
  *
@@ -38,9 +44,9 @@
  *
  */
 #ifndef GKCONFIG_H 
-#define GKCONFIG_H "@(#) $Id: gkconfig.h,v 1.6 2007/11/27 12:57:27 willamowius Exp $"
+#define GKCONFIG_H "@(#) $Id: gkconfig.h,v 1.8 2008/05/23 10:47:01 willamowius Exp $"
 
-#include "pwlib_compat.h"
+#include "config.h"
 
 class GatekeeperConfig : public PConfig
 {
@@ -66,10 +72,18 @@ public:
 
 	    @return list of all section names.
 	*/
+#ifdef hasPConfigArray
+    virtual PStringArray GetSections() const;
+#else
     virtual PStringList GetSections() const;
+#endif
 
 	/** Get a list of all the keys in the section. */
+#ifdef hasPConfigArray
+	virtual PStringArray GetKeys(
+#else
 	virtual PStringList GetKeys(
+#endif
 		const PString& theSection /// Section to use instead of the default.
 		) const;
 
