@@ -11,6 +11,24 @@
  * with the OpenH323 library.
  *
  * $Log: gksql.cxx,v $
+ * Revision 1.19  2008/11/21 13:47:21  willamowius
+ * remove special case for pool size = 1: it didn't handle the case that connections now get deleted on error
+ *
+ * Revision 1.18  2008/09/05 13:44:14  zvision
+ * GetInfo implemented for SQL acct/auth modules
+ *
+ * Revision 1.17  2008/09/04 08:19:04  zvision
+ * SQL reconnect thread safety fixes
+ *
+ * Revision 1.16  2008/08/29 08:39:19  zvision
+ * Fixed missing lock around Connect
+ *
+ * Revision 1.15  2008/07/10 08:03:17  willamowius
+ * avoid gcc 4.3.x warnings
+ *
+ * Revision 1.14  2008/04/02 22:32:22  willamowius
+ * auto-reconnect on database errors
+ *
  * Revision 1.13  2006/04/14 13:56:19  willamowius
  * call failover code merged
  *
@@ -152,15 +170,18 @@ bool GkSQLConnection::Initialize(
 		return false;
 	}
 	
-	if (!Connect())
+	if (!Connect()) {
 		PTRACE(1, GetName() << "\tDatabase connection failed, will rety later");
+	}
 	
 	return true;
 }
 
 bool GkSQLConnection::Connect()
 {
-	for (PINDEX i = 0; i < m_minPoolSize; i++) {
+	PWaitAndSignal lock(m_connectionsMutex);
+	
+	for (PINDEX i = m_idleConnections.size(); i < m_minPoolSize; ++i) {
 		SQLConnPtr connptr = CreateNewConnection(i);
 		if (connptr != NULL)
 			m_idleConnections.push_back(connptr);
@@ -184,6 +205,26 @@ bool GkSQLConnection::Connect()
 	}
 }
 
+void GkSQLConnection::Disconnect()
+{
+	PWaitAndSignal lock(m_connectionsMutex);
+	
+	// disconnect/delete all connections
+	PTRACE(3, GetName() << "\tDisconnecting all SQL connections in pool");
+	for(iterator Iter = m_idleConnections.begin();
+		Iter != m_idleConnections.end(); Iter++) {
+		delete *Iter;
+	}
+	m_idleConnections.clear();
+//	for(iterator Iter = m_busyConnections.begin();
+//		Iter != m_busyConnections.end(); Iter++) {
+//		delete *Iter;
+//	}
+//	m_busyConnections.clear();
+	// TODO:: delete waiting ?
+	m_connected = false;
+}
+
 GkSQLConnection::~GkSQLConnection()
 {
 	const PTime timeStart;
@@ -221,8 +262,9 @@ GkSQLConnection::~GkSQLConnection()
 	m_idleConnections.clear();
 
 	PTRACE(5, GetName() << "\tConnection pool cleanup finished");
-	if (!m_busyConnections.empty())
+	if (!m_busyConnections.empty()) {
 		PTRACE(1, GetName() << "\tConnection cleanup finished with " << m_busyConnections.size() << " active connections");
+	}
 }
 
 bool GkSQLConnection::AcquireSQLConnection(
@@ -232,9 +274,10 @@ bool GkSQLConnection::AcquireSQLConnection(
 {
 	if (m_destroying)
 		return false;
-		
+
 	if (!m_connected) {
 		PTRACE(2, GetName() << "\tAttempting to reconnect to the database");
+		Disconnect();
 		if (!Connect()) {
 			PTRACE(2, GetName() << "\tFailed to reconnect to the database");
 			return false;
@@ -243,104 +286,94 @@ bool GkSQLConnection::AcquireSQLConnection(
 	
 	const PTime timeStart;
 	connptr = NULL;
-	
-	// special case (no pool) for fast execution
-	if (m_minPoolSize == 1 && m_maxPoolSize == 1) {
-		if (!(timeout == -1 ? (m_connectionsMutex.Wait(), true) : m_connectionsMutex.Wait(timeout))) {
-			PTRACE(2, GetName() << "\tQuery timed out waiting " << timeout << "ms for the connection");
-			return false;
-		}
-		if (!m_destroying)
-			connptr = m_idleConnections.front();
-	} else {
-		bool waiting = false;
-	
-		// wait for an idle connection or timeout
-		do {
-			if (!waiting) {
-				PWaitAndSignal lock(m_connectionsMutex);
-		
-				if (m_destroying)
-					break;
+	bool waiting = false;
 
-				// grab an idle connection if available or add itself
-				// to the list of waiting requests
-				if (!m_idleConnections.empty()) {
-					connptr = m_idleConnections.front();
-					m_idleConnections.pop_front();
-					m_busyConnections.push_front(connptr);
-				} else {
-					m_waitingRequests.push_back(&connptr);
-					waiting = true;
-				}
-			}
-		
-			if (connptr == NULL && timeout != 0 && !m_destroying)
-				m_connectionAvailable.Wait(min(250L,timeout));
-		
-			if (connptr == NULL && timeout >= 0)
-				if ((PTime()-timeStart).GetMilliSeconds() >= timeout)
-					break;
-		} while (connptr == NULL && !m_destroying);
-
-		if (connptr == NULL || m_destroying) {
+	// wait for an idle connection or timeout
+	do {
+		if (!waiting) {
 			PWaitAndSignal lock(m_connectionsMutex);
-			m_waitingRequests.remove(&connptr);
-			if (connptr) {
-				m_idleConnections.push_back(connptr);
-				m_busyConnections.remove(connptr);
+
+			if (m_destroying)
+				break;
+
+			// grab an idle connection if available or add itself
+			// to the list of waiting requests
+			if (!m_idleConnections.empty()) {
+				connptr = m_idleConnections.front();
+				m_idleConnections.pop_front();
+				m_busyConnections.push_front(connptr);
+			} else {
+				m_waitingRequests.push_back(&connptr);
+				waiting = true;
 			}
-			PTRACE(2, GetName() << "\tQuery timed out waiting for idle connection");
-			return false;
 		}
+
+		if (connptr == NULL && timeout != 0 && !m_destroying)
+			m_connectionAvailable.Wait(min(250L,timeout));
+
+		if (connptr == NULL && timeout >= 0)
+			if ((PTime()-timeStart).GetMilliSeconds() >= timeout)
+				break;
+	} while (connptr == NULL && !m_destroying);
+
+	if (connptr == NULL || m_destroying) {
+		PWaitAndSignal lock(m_connectionsMutex);
+		m_waitingRequests.remove(&connptr);
+		if (connptr) {
+			m_idleConnections.push_back(connptr);
+			m_busyConnections.remove(connptr);
+		}
+		PTRACE(2, GetName() << "\tQuery timed out waiting for idle connection");
+		return false;
 	}
 
 	return connptr != NULL;
 }
 
 void GkSQLConnection::ReleaseSQLConnection(
-	SQLConnPtr& connptr
+	SQLConnPtr& connptr,
+	bool deleteFromPool
 	)
 {
 	if (connptr == NULL)
 		return;
-	// special case (no pool) for fast execution
-	if (m_minPoolSize == 1 && m_maxPoolSize == 1)
-		m_connectionsMutex.Signal();
-	else {
-		// mark the connection as idle or give it to the first waiting request
-		{
-			PWaitAndSignal lock(m_connectionsMutex);
-		
-			// remove itself from the list of waiting requests
-			m_waitingRequests.remove(&connptr);
-		
-			witerator iter = m_waitingRequests.begin();
-			witerator end = m_waitingRequests.end();
-		
-			// find a waiting requests that has not been given a connection yet
-			while (iter != end) {
-				// check if SQLConnPtr* is not NULL and if SQLConnPtr is empty (NULL)
-				if (*iter && *(*iter) == NULL)
-					break;
-				iter++;
-			}
-			if (iter != end && !m_destroying) {
-				// do not remove itself from the list of busy connections
-				// just move the connection to the waiting request
-				*(*iter) = connptr;
-			} else {
-				// move the connection to the list of idle connections
-				m_busyConnections.remove(connptr);
+
+	// mark the connection as idle or give it to the first waiting request
+	{
+		PWaitAndSignal lock(m_connectionsMutex);
+
+		// remove itself from the list of waiting requests
+		m_waitingRequests.remove(&connptr);
+
+		witerator iter = m_waitingRequests.begin();
+		witerator end = m_waitingRequests.end();
+
+		// find a waiting requests that has not been given a connection yet
+		while (iter != end) {
+			// check if SQLConnPtr* is not NULL and if SQLConnPtr is empty (NULL)
+			if (*iter && *(*iter) == NULL)
+				break;
+			iter++;
+		}
+		if (iter != end && !m_destroying && !deleteFromPool) {
+			// do not remove itself from the list of busy connections
+			// just move the connection to the waiting request
+			*(*iter) = connptr;
+		} else {
+			// move the connection to the list of idle connections
+			m_busyConnections.remove(connptr);
+			if (deleteFromPool)
+				delete connptr;
+			else
 				m_idleConnections.push_back(connptr);
-			}
-		
-			connptr = NULL;
 		}
 
-		// wake up any threads waiting for an idle connection
-		m_connectionAvailable.Signal();
+		connptr = NULL;
 	}
+
+	// wake up any threads waiting for an idle connection
+	if (!deleteFromPool)
+		m_connectionAvailable.Signal();
 }
 
 GkSQLResult* GkSQLConnection::ExecuteQuery(
@@ -361,7 +394,7 @@ GkSQLResult* GkSQLConnection::ExecuteQuery(
 			PTRACE(5, GetName() << "\tExecuting query: " << queryStr);
 			result = ExecuteQuery(connptr, queryStr, timeout);
 		}
-		ReleaseSQLConnection(connptr);
+		ReleaseSQLConnection(connptr, !m_connected);
 		return result;
 	} else {
 		PTRACE(2, GetName() << "\tQuery failed - no idle connection in the pool");
@@ -387,7 +420,7 @@ GkSQLResult* GkSQLConnection::ExecuteQuery(
 			PTRACE(5, GetName() << "\tExecuting query: " << finalQueryStr);
 			result = ExecuteQuery(connptr, finalQueryStr, timeout);
 		}
-		ReleaseSQLConnection(connptr);
+		ReleaseSQLConnection(connptr, !m_connected);
 		return result;
 	} else {
 		PTRACE(2, GetName() << "\tQuery failed - no idle connection in the pool");
@@ -522,6 +555,20 @@ PString GkSQLConnection::ReplaceQueryParams(
 	return finalQuery;
 }
 
+void GkSQLConnection::GetInfo(
+	Info &info /// filled with SQL connection state information upon return
+	)
+{
+	PWaitAndSignal lock(m_connectionsMutex);
+
+	info.m_connected = m_connected;
+	info.m_minPoolSize = m_minPoolSize;
+	info.m_maxPoolSize = m_maxPoolSize;
+	info.m_idleConnections = m_idleConnections.size();
+	info.m_busyConnections = m_busyConnections.size();
+	info.m_waitingRequests = m_waitingRequests.size();
+}
+
 GkSQLConnection::SQLConnWrapper::~SQLConnWrapper()
 {
 }
