@@ -11,6 +11,18 @@
  * with the OpenH323 library.
  *
  * $Log: gksql.h,v $
+ * Revision 1.15  2008/09/05 13:44:14  zvision
+ * GetInfo implemented for SQL acct/auth modules
+ *
+ * Revision 1.14  2008/09/04 08:19:04  zvision
+ * SQL reconnect thread safety fixes
+ *
+ * Revision 1.13  2008/04/03 10:24:20  willamowius
+ * new header config.h for configuration details
+ *
+ * Revision 1.12  2008/04/02 22:32:22  willamowius
+ * auto-reconnect on database errors
+ *
  * Revision 1.11  2007/09/10 18:13:48  willamowius
  * clean up sql driver interface and remove unused methods from all drivers
  *
@@ -46,14 +58,14 @@
  *
  */
 #ifndef GKSQL_H
-#define GKSQL_H "@(#) $Id: gksql.h,v 1.11 2007/09/10 18:13:48 willamowius Exp $"
+#define GKSQL_H "@(#) $Id: gksql.h,v 1.15 2008/09/05 13:44:14 zvision Exp $"
 
 #include <list>
 #include <map>
 #include <vector>
 #include "name.h"
 #include "factory.h"
-#include "pwlib_compat.h"
+#include "config.h"
 
 /** Abstract base class that encapsulates SQL query result.
     Backend specific operations are performed by derived classes.
@@ -140,6 +152,15 @@ protected:
 class GkSQLConnection : public NamedObject
 {
 public:
+	struct Info {
+		bool m_connected;
+		unsigned m_idleConnections;
+		unsigned m_busyConnections;
+		unsigned m_waitingRequests;
+		int m_minPoolSize;
+		int m_maxPoolSize;
+	};
+	
 	GkSQLConnection(
 		/// name to use in the log
 		const char* name = "SQL"
@@ -240,6 +261,11 @@ public:
 		long timeout = -1
 		);
 
+	/// Get information about SQL connection state
+	void GetInfo(
+		Info &info /// filled with SQL connection state information upon return
+		);
+		
 #if defined(_WIN32) && (_MSC_VER < 1300)
 public:
 #else
@@ -306,7 +332,8 @@ protected:
 	    AcquireSQLConnection.
 	*/
 	void ReleaseSQLConnection(
-		SQLConnPtr& connptr /// connection to release (mark as idle)
+		SQLConnPtr& connptr, /// connection to release (mark as idle)
+		bool deleteFromPool = false /// true to delete the connection (a broken connection)
 		);
 
 	/** Execute the query using specified SQL connection.
@@ -390,8 +417,12 @@ private:
 	    True if at least one database connection has been established.
 	*/
 	bool Connect();
-	
+
 protected:
+	/** Disconnect connection pool from DB on connection error
+	*/
+	void Disconnect();
+	
 	/// filled with the actual host from m_hosts the database connection is made to
 	PString m_host;
 	/// database port to connect to
@@ -427,6 +458,7 @@ private:
 	/// set to true when destructor is being invoked
 	bool m_destroying;
 	/// remain false while connection to the database not yet established
+	/// reset to false on disconnect or error during operation -> reconnect
 	bool m_connected;
 };
 
