@@ -11,8 +11,20 @@
  * with the OpenH323 library.
  *
  * $Log: gksql_firebird.cxx,v $
- * Revision 1.8.2.1  2008/05/20 17:44:47  willamowius
- * more braces to avoid warnings with gcc 4.3.0
+ * Revision 1.13  2008/11/08 18:31:41  willamowius
+ * TODO: replace all isc_interprete() with fb_interpret()
+ *
+ * Revision 1.12  2008/05/20 18:13:51  willamowius
+ * more braces to avoid gcc 4.3.0 warnings
+ *
+ * Revision 1.11  2008/04/18 14:37:28  willamowius
+ * never include gnugkbuildopts.h directly, always include config.h
+ *
+ * Revision 1.10  2008/04/18 13:14:11  shorne
+ * Fixes for auto-configure on windows
+ *
+ * Revision 1.9  2008/04/02 22:32:22  willamowius
+ * auto-reconnect on database errors
  *
  * Revision 1.8  2007/09/10 18:13:48  willamowius
  * clean up sql driver interface and remove unused methods from all drivers
@@ -39,6 +51,9 @@
  * Firebird SQL driver
  *
  */
+
+#include "config.h"
+
 #if HAS_FIREBIRD
 
 #if defined(_WIN32) && (_MSC_VER <= 1200)
@@ -51,6 +66,10 @@
 #include <ibase.h>
 #include "gksql.h"
 
+#ifdef _WIN32
+#pragma comment( lib, FIREBIRD_LIBRARY )
+#endif
+
 namespace {
 PString XSQLVARToPString(XSQLVAR *sqlvar)
 {
@@ -352,6 +371,7 @@ bool GkIBSQLResult::FetchRow(
 			if (errcode == -999) {
 				errcode = status[1];
 				long *pvector = status;
+				// TODO: replace all isc_interprete() with fb_interpret()
 				errormsg[isc_interprete(errormsg, &pvector)] = 0;
 			} else {
 				strcpy(errormsg, "SQL:");
@@ -527,6 +547,7 @@ GkSQLResult* GkIBSQLConnection::ExecuteQuery(
 			strcpy(errormsg, "SQL:");
 			isc_sql_interprete(static_cast<short>(errorcode), errormsg, 512 - 4);
 		}
+		Disconnect();
 		return new GkIBSQLResult(errorcode, errormsg, tr);
 	}
 
@@ -547,6 +568,7 @@ GkSQLResult* GkIBSQLConnection::ExecuteQuery(
 			strcpy(errormsg, "SQL:");
 			isc_sql_interprete(static_cast<short>(errorcode), errormsg, 512 - 4);
 		}
+		Disconnect();
 		return new GkIBSQLResult(errorcode, errormsg, tr, stmt);
 	}
 	
@@ -571,6 +593,7 @@ GkSQLResult* GkIBSQLConnection::ExecuteQuery(
 			}
 			delete [] reinterpret_cast<char*>(result);
 			result = NULL;
+			Disconnect();
 			return new GkIBSQLResult(errorcode, errormsg, tr, stmt);
 		}
 	}
@@ -588,6 +611,7 @@ GkSQLResult* GkIBSQLConnection::ExecuteQuery(
 		}
 		delete [] reinterpret_cast<char*>(result);
 		result = NULL;
+		Disconnect();
 		return new GkIBSQLResult(errorcode, errormsg, tr, stmt);
 	}
 
