@@ -17,22 +17,20 @@
 //////////////////////////////////////////////////////////////////
 
 #ifndef JOB_H
-#define JOB_H "@(#) $Id: job.h,v 1.8 2008/03/13 09:40:37 zvision Exp $"
+#define JOB_H "@(#) $Id: job.h,v 1.10 2008/07/23 13:02:17 willamowius Exp $"
 
 #include "name.h"
 
 /** The base abstract class that represents job objects.
     This class implements the way to execute the job.
     Derived classes implement actual jobs (override Run()).
-	Jobs are executed and managed by internal Job managment system
+	Jobs are executed and managed by the internal Job managment system
 	consisting of the singleton Agent object and Worker threads
 	that accept and execute new Jobs.
 	
 	Specialized Job examples are:
     Jobs - executes series of Tasks
     RegularJob - executes a Task again and again until stopped
-    SimpleJob - calls some funtion with one pointer-type argument
-    SimpleJobA - calls some funtion with two arguments : pointer-type and reference
     SimpleClassJob - calls a member function of some class that takes no arguments
     SimpleClassJobA - calls a member function of some class that takes one 
 	                  argument of reference type
@@ -214,29 +212,6 @@ private:
 };
 
 
-template<class F, class T>
-class SimpleJob : public Job {
-public:
-	SimpleJob(const F & _f, T *_t) : f(_f), t(_t) {}
-	virtual void Run() { f(t); }
-
-private:
-	const F f;
-	T *t;
-};
-
-template<class F, class T, class A>
-class SimpleJobA : public Job {
-public:
-	SimpleJobA(const F & _f, T *_t, const A & _a) : f(_f), t(_t), a(_a) {}
-	virtual void Run() { f(t, a); }
-
-private:
-	const F f;
-	T *t;
-	A a;
-};
-
 template<class T>
 class SimpleClassJob : public Job {
 public:
