@@ -12,6 +12,18 @@
  * with the OpenH323 library.
  *
  * $Log: radauth.cxx,v $
+ * Revision 1.46  2008/09/15 08:42:09  zvision
+ * Allow outbound number rewrite by SQL/RADIUS modules
+ *
+ * Revision 1.45  2008/08/13 13:10:52  zvision
+ * h323-redirect-ip crash when no h323-redirect-number fixed
+ *
+ * Revision 1.44  2008/04/18 14:37:28  willamowius
+ * never include gnugkbuildopts.h directly, always include config.h
+ *
+ * Revision 1.43  2008/04/18 13:14:11  shorne
+ * Fixes for auto-configure on windows
+ *
  * Revision 1.42  2008/01/30 19:00:31  willamowius
  * fix bug introduced when renaming variables
  *
@@ -209,7 +221,9 @@
  * Initial revision
  *
  */
- 
+
+#include "config.h"
+
 #if HAS_RADIUS
 
 #if defined(_WIN32) && (_MSC_VER <= 1200)
@@ -816,10 +830,16 @@ int RadAuthBase::Check(
 			value = attr->AsCiscoString();
 			if (!value) {
 				numbersToDial = value.Tokenise("; \t", FALSE);
-				authData.SetRouteToAlias(numbersToDial[0]);
-				PTRACE(5, "RADAUTH\t" << GetName() << " ARQ check redirect "
-					"to the number " << value
-					);
+				if (numbersToDial.GetSize() > 0) {
+					authData.SetRouteToAlias(numbersToDial[0]);
+					PTRACE(5, "RADAUTH\t" << GetName() << " ARQ check redirect "
+						"to the number " << value
+						);
+				} else {
+					PTRACE(1, "RADAUTH\t" << GetName()
+						<< " invalid ARQ check redirect numbers list: " << value
+						);
+				}
 			}
 		}
 	}
@@ -843,7 +863,14 @@ int RadAuthBase::Check(
 						route.m_destEndpoint = RegistrationTable::Instance()->FindBySignalAdr(
 							SocketToH225TransportAddr(raddr, rport)
 							);
-						route.m_destNumber = (i < numbersToDial.GetSize()) ? numbersToDial[i] : numbersToDial[numbersToDial.GetSize() - 1];
+						if (numbersToDial.GetSize() > 0) {
+							route.m_destNumber = (i < numbersToDial.GetSize()) ? numbersToDial[i] : numbersToDial[numbersToDial.GetSize() - 1];
+							PINDEX pos = route.m_destNumber.Find('=');
+							if (pos != P_MAX_INDEX) {
+								route.m_destOutNumber = route.m_destNumber.Mid(pos + 1);
+								route.m_destNumber = route.m_destNumber.Left(pos);
+							}
+						}
 						authData.m_destinationRoutes.push_back(route);
 						PTRACE(5, "RADAUTH\t" << GetName() << " ARQ check redirect "
 							"to the address " << route.AsString()
@@ -1073,10 +1100,16 @@ int RadAuthBase::Check(
 			value = attr->AsCiscoString();
 			if (!value) {
 				numbersToDial = value.Tokenise("; \t", FALSE);
-				authData.SetRouteToAlias(numbersToDial[0]);
-				PTRACE(5, "RADAUTH\t" << GetName() << " ARQ check redirect "
-					"to the number " << value
-					);
+				if (numbersToDial.GetSize() > 0) {
+					authData.SetRouteToAlias(numbersToDial[0]);
+					PTRACE(5, "RADAUTH\t" << GetName() << " ARQ check redirect "
+						"to the number " << value
+						);
+				} else {
+					PTRACE(1, "RADAUTH\t" << GetName()
+						<< " invalid ARQ check redirect numbers list: " << value
+						);
+				}
 			}
 		}
 	}
@@ -1100,7 +1133,14 @@ int RadAuthBase::Check(
 						route.m_destEndpoint = RegistrationTable::Instance()->FindBySignalAdr(
 							SocketToH225TransportAddr(raddr, rport)
 							);
-						route.m_destNumber = (i < numbersToDial.GetSize()) ? numbersToDial[i] : numbersToDial[numbersToDial.GetSize() - 1];
+						if (numbersToDial.GetSize() > 0) {
+							route.m_destNumber = (i < numbersToDial.GetSize()) ? numbersToDial[i] : numbersToDial[numbersToDial.GetSize() - 1];
+							PINDEX pos = route.m_destNumber.Find('=');
+							if (pos != P_MAX_INDEX) {
+								route.m_destOutNumber = route.m_destNumber.Mid(pos + 1);
+								route.m_destNumber = route.m_destNumber.Left(pos);
+							}
+						}
 						authData.m_destinationRoutes.push_back(route);
 						PTRACE(5, "RADAUTH\t" << GetName() << " Setup check redirect "
 							"to the address " << route.AsString()
