@@ -11,6 +11,24 @@
  * with the OpenH323 library.
  *
  * $Log: radproto.cxx,v $
+ * Revision 1.42  2008/12/03 18:52:49  willamowius
+ * new RadAcct attribute: RewriteE164 (thanks Tusar)
+ *
+ * Revision 1.41  2008/07/10 08:18:33  willamowius
+ * avoid gcc 4.3.x warnings
+ *
+ * Revision 1.40  2008/05/18 20:40:56  willamowius
+ * put codec in RadAcct stop event (Thanks Tusar)
+ *
+ * Revision 1.39  2008/04/18 14:37:28  willamowius
+ * never include gnugkbuildopts.h directly, always include config.h
+ *
+ * Revision 1.38  2008/04/18 13:14:11  shorne
+ * Fixes for auto-configure on windows
+ *
+ * Revision 1.37  2008/04/03 12:03:35  willamowius
+ * set radius release-source attribute in stop accounting packet, like %r in sqlacct
+ *
  * Revision 1.36  2008/03/18 15:51:13  zvision
  * Delete socket when id cannot be generated
  *
@@ -164,6 +182,9 @@
  * Initial revision
  *
  */
+
+#include "config.h"
+
 #if HAS_RADIUS
 
 #if defined(_WIN32) && (_MSC_VER <= 1200)
@@ -270,6 +291,9 @@ struct CiscoAttrName {
 	CiscoAttrName(CISCO_ATTR_NAME("h323-voice-quality"), RadiusAttr::CiscoVSA_h323_voice_quality),
 	CiscoAttrName(CISCO_ATTR_NAME("h323-incoming-conf-id"), RadiusAttr::CiscoVSA_h323_incoming_conf_id),
 	CiscoAttrName(CISCO_ATTR_NAME("h323-preferred-lang"), RadiusAttr::CiscoVSA_h323_preferred_lang),
+	CiscoAttrName(CISCO_ATTR_NAME("release-source"),RadiusAttr::CiscoVSA_release_source),
+	CiscoAttrName(CISCO_ATTR_NAME("preferred-codec"),RadiusAttr::CiscoVSA_preferred_codec),
+	CiscoAttrName(CISCO_ATTR_NAME("rewritten-e164-num"),RadiusAttr::CiscoVSA_rewritten_e164_num),
 	CiscoAttrName(NULL, 0, 0)
 };
 
@@ -1653,12 +1677,13 @@ bool RadiusSocket::MakeRequest(
 
 	m_writeMutex.Wait();
 	bool result = WriteTo(request, length, serverAddress, serverPort);
-	if (!result)
+	if (!result) {
 		PTRACE(5, "RADIUS\tError sending UDP packet ("
 			<< GetErrorCode(LastWriteError) << '/'
 			<< GetErrorNumber(LastWriteError) << ": "
 			<< GetErrorText(LastWriteError) << " (id:" << (PINDEX)id << ')'
 			);
+	}
 	m_writeMutex.Signal();
 	
 	if (result)
@@ -1903,13 +1928,14 @@ RadiusClient::RadiusClient(
 {
 	GetServersFromString(servers);
 	
-	if (!address)
-		if (!PIPSocket::IsLocalHost(address))
+	if (!address) {
+		if (!PIPSocket::IsLocalHost(address)) {
 			PTRACE(1, "RADIUS\tSpecified local client address " << address
-				<< " is not bound to any local interface"
-				);
-		else
+				<< " is not bound to any local interface");
+		} else {
 			PIPSocket::GetHostAddress(address, m_localAddress);
+		}
+	}
 
 #if PTRACING
 	if (PTrace::CanTrace(4)) {
@@ -1955,13 +1981,14 @@ RadiusClient::RadiusClient(
 		
 	const PString addr = config.GetString(sectionName, "LocalInterface", "");
 	
-	if (!addr)
-		if (!PIPSocket::IsLocalHost(addr))
+	if (!addr) {
+		if (!PIPSocket::IsLocalHost(addr)) {
 			PTRACE(2, "RADIUS\tSpecified local client address '" << addr 
-				<< "' is not bound to any local interface"
-				);
-		else
+				<< "' is not bound to any local interface");
+		} else {
 			PIPSocket::GetHostAddress(addr, m_localAddress);
+		}
+	}
 
 	// parse port range (if it does exist)
 	const PStringArray s
