@@ -12,6 +12,18 @@
  * with the OpenH323 library.
  *
  * $Log: radproto.h,v $
+ * Revision 1.20  2008/12/03 18:52:49  willamowius
+ * new RadAcct attribute: RewriteE164 (thanks Tusar)
+ *
+ * Revision 1.19  2008/05/18 20:40:56  willamowius
+ * put codec in RadAcct stop event (Thanks Tusar)
+ *
+ * Revision 1.18  2008/04/03 12:03:35  willamowius
+ * set radius release-source attribute in stop accounting packet, like %r in sqlacct
+ *
+ * Revision 1.17  2008/04/03 10:24:20  willamowius
+ * new header config.h for configuration details
+ *
  * Revision 1.16  2008/01/09 00:44:07  willamowius
  * remove unused code
  *
@@ -96,11 +108,11 @@
 #if HAS_RADIUS
 
 #ifndef __RADPROTO_H
-#define __RADPROTO_H "@(#) $Id: radproto.h,v 1.16 2008/01/09 00:44:07 willamowius Exp $"
+#define __RADPROTO_H "@(#) $Id: radproto.h,v 1.20 2008/12/03 18:52:49 willamowius Exp $"
 
 #include <vector>
 #include <ptlib/sockets.h>
-#include "pwlib_compat.h"
+#include "config.h"
 
 class PRandom;
 class PMessageDigest5;
@@ -222,7 +234,9 @@ public:
 		CiscoVSA_h323_return_code = 103, CiscoVSA_h323_redirect_number = 106,
 		CiscoVSA_h323_preferred_lang = 107, 
 		CiscoVSA_h323_redirect_ip_address = 108, 
-		CiscoVSA_h323_billing_model = 109, CiscoVSA_h323_currency = 110
+		CiscoVSA_h323_billing_model = 109, CiscoVSA_h323_currency = 110,
+		CiscoVSA_release_source = 115, CiscoVSA_preferred_codec = 116,
+		CiscoVSA_rewritten_e164_num = 117
 	};
 	
 	/** Construct uninitialized attribute. It should be initialized
