@@ -2,7 +2,7 @@
 //
 // slist.h
 //
-// A Single Linked List Template
+// A Single Linked List Template   *** LEGACY - use STL for new code ***
 //
 // Copyright (c) Citron Network Inc. 2003
 //
@@ -17,7 +17,7 @@
 //////////////////////////////////////////////////////////////////
 
 #ifndef SLIST_H
-#define SLIST_H "@(#) $Id: slist.h,v 1.3 2006/04/14 13:56:19 willamowius Exp $"
+#define SLIST_H "@(#) $Id: slist.h,v 1.4 2008/04/18 15:53:53 willamowius Exp $"
 
 #include "factory.h"
 
@@ -29,9 +29,6 @@ public:
 	SList() : m_next(0) {}
 	virtual ~SList() = 0;
 
-	T* GetNext() { return m_next; }
-	const T* GetNext() const { return m_next; }
-
 	static T *Create(const PStringArray &);
 	
 protected:
