@@ -11,6 +11,12 @@
  * with the OpenH323 library.
  *
  * $Log: sqlacct.cxx,v $
+ * Revision 1.16  2008/09/05 13:44:14  zvision
+ * GetInfo implemented for SQL acct/auth modules
+ *
+ * Revision 1.15  2008/07/10 08:07:54  willamowius
+ * avoid gcc 4.3.x warnings
+ *
  * Revision 1.14  2007/09/28 22:20:23  willamowius
  * cleanup includes
  *
@@ -136,29 +142,27 @@ SQLAcct::SQLAcct(
 		PTRACE(4, "GKACCT\t" << GetName() << " start query: " << m_startQuery);
 	
 	m_startQueryAlt = cfg->GetString(cfgSec, "StartQueryAlt", "");
-	if (!m_startQueryAlt) 
-		PTRACE(4, "GKACCT\t" << GetName() << " alternative start query: " 
-			<< m_startQueryAlt
-			);
+	if (!m_startQueryAlt) {
+		PTRACE(4, "GKACCT\t" << GetName() << " alternative start query: " << m_startQueryAlt);
+	}
 
 	m_updateQuery = cfg->GetString(cfgSec, "UpdateQuery", "");
 	if (m_updateQuery.IsEmpty() 
 		&& (GetEnabledEvents() & GetSupportedEvents() & (AcctUpdate | AcctConnect)) != 0) {
 		PTRACE(0, "GKACCT\t" << GetName() << " module creation failed: "
-			"no update query configured"
-			);
+			"no update query configured");
 		PTRACE(0, "GKACCT\tFATAL: Shutting down");
 		RasServer::Instance()->Stop();
 		return;
-	} else
+	} else {
 		PTRACE(4, "GKACCT\t" << GetName() << " update query: " << m_updateQuery);
+	}
 
 	m_stopQuery = cfg->GetString(cfgSec, "StopQuery", "");
 	if (m_stopQuery.IsEmpty() 
 		&& (GetEnabledEvents() & GetSupportedEvents() & AcctStop) == AcctStop) {
 		PTRACE(0, "GKACCT\t" << GetName() << " module creation failed: "
-			"no stop query configured"
-			);
+			"no stop query configured");
 		PTRACE(0, "GKACCT\tFATAL: Shutting down");
 		RasServer::Instance()->Stop();
 		return;
@@ -166,10 +170,10 @@ SQLAcct::SQLAcct(
 		PTRACE(4, "GKACCT\t" << GetName() << " stop query: " << m_stopQuery);
 	
 	m_stopQueryAlt = cfg->GetString(cfgSec, "StopQueryAlt", "");
-	if (!m_stopQueryAlt) 
+	if (!m_stopQueryAlt) {
 		PTRACE(4, "GKACCT\t" << GetName() << " alternative stop query: " 
-			<< m_stopQueryAlt
-			);
+			<< m_stopQueryAlt);
+	}
 
 	vector<PIPSocket::Address> interfaces;
 	Toolkit::Instance()->GetGKHome(interfaces);
@@ -242,11 +246,11 @@ GkAcctLogger::Status SQLAcct::Log(
 	std::map<PString, PString> params;
 	SetupAcctParams(params, call, m_timestampFormat);
 	GkSQLResult* result = m_sqlConn->ExecuteQuery(query, params);
-	if (result == NULL)
+	if (result == NULL) {
 		PTRACE(2, "GKACCT\t" << GetName() << " failed to store accounting "
 			"data (event: " << evt << ", call: " << callNumber 
-			<< "): timeout or fatal error"
-			);
+			<< "): timeout or fatal error");
+	}
 	
 	if (result) {
 		if (result->IsValid()) {
@@ -278,11 +282,11 @@ GkAcctLogger::Status SQLAcct::Log(
 				);
 		else {
 			if (result->IsValid()) {
-				if (result->GetNumRows() < 1)
+				if (result->GetNumRows() < 1) {
 					PTRACE(4, "GKACCT\t" << GetName() << " failed to store accounting "
 						"data (event: " << evt << ", call: " << callNumber 
-						<< "): no rows have been updated"
-						);
+						<< "): no rows have been updated");
+				}
 			} else
 				PTRACE(2, "GKACCT\t" << GetName() << " failed to store accounting "
 					"data (event: " << evt << ", call: " << callNumber 
@@ -297,6 +301,28 @@ GkAcctLogger::Status SQLAcct::Log(
 	return succeeded ? Ok : Fail;
 }
 
+PString SQLAcct::GetInfo()
+{
+	PString result;
+	
+	if (m_sqlConn == NULL)
+		result += "  No SQL connection available\r\n";
+	else {
+		GkSQLConnection::Info info;
+		m_sqlConn->GetInfo(info);
+		result += "  Connected to an SQL Backend: " + PString(info.m_connected ? "Yes" : "No") + "\r\n";
+		result += "  Min Connection Pool Size:    " + PString(info.m_minPoolSize) + "\r\n";
+		result += "  Max Connection Pool Size:    " + PString(info.m_maxPoolSize) + "\r\n";
+		result += "  Idle Connections:            " + PString(info.m_idleConnections) + "\r\n";
+		result += "  Busy Connections::           " + PString(info.m_busyConnections) + "\r\n";
+		result += "  Waiting Requests:            " + PString(info.m_waitingRequests) + "\r\n";
+	}
+	
+	result += ";\r\n";
+	
+	return result;
+}
+
 namespace {
 GkAcctLoggerCreator<SQLAcct> SQLAcctLoggerCreator("SQLAcct");
 }
