@@ -3,7 +3,7 @@
  *
  * SQL authentication/authorization modules for GNU Gatekeeper
  *
- * $Id: sqlauth.cxx,v 1.12 2007/11/25 23:23:40 willamowius Exp $
+ * $Id: sqlauth.cxx,v 1.19 2008/11/19 12:47:55 zvision Exp $
  *
  * Copyright (c) 2004, Michal Zygmuntowicz
  *
@@ -51,6 +51,8 @@ public:
 	
 	virtual ~SQLPasswordAuth();
 
+	virtual PString GetInfo();
+	
 protected:
 	/** Override from SimplePasswordAuth.
 	
@@ -88,6 +90,8 @@ public:
 	
 	virtual ~SQLAliasAuth();
 
+	virtual PString GetInfo();
+
 protected:
 	/** Get auth condition string for the given alias. 
 	    This implementation searches the SQL database for the string.
@@ -184,6 +188,8 @@ public:
 		SetupAuthData& authData
 		);
 
+	virtual PString GetInfo();
+
 private:
 	SQLAuth();
 	SQLAuth(const SQLAuth&);
@@ -342,6 +348,28 @@ bool SQLPasswordAuth::GetPassword(
 	return true;
 }
 
+PString SQLPasswordAuth::GetInfo()
+{
+	PString result;
+	
+	if (m_sqlConn == NULL)
+		result += "  No SQL connection available\r\n";
+	else {
+		GkSQLConnection::Info info;
+		m_sqlConn->GetInfo(info);
+		result += "  Connected to an SQL Backend: " + PString(info.m_connected ? "Yes" : "No") + "\r\n";
+		result += "  Min Connection Pool Size:    " + PString(info.m_minPoolSize) + "\r\n";
+		result += "  Max Connection Pool Size:    " + PString(info.m_maxPoolSize) + "\r\n";
+		result += "  Idle Connections:            " + PString(info.m_idleConnections) + "\r\n";
+		result += "  Busy Connections::           " + PString(info.m_busyConnections) + "\r\n";
+		result += "  Waiting Requests:            " + PString(info.m_waitingRequests) + "\r\n";
+	}
+	
+	result += ";\r\n";
+	
+	return result;
+}
+
 SQLAliasAuth::SQLAliasAuth(
 	const char* authName
 	)
@@ -414,6 +442,27 @@ bool SQLAliasAuth::GetAuthConditionString(
 	return true;
 }
 
+PString SQLAliasAuth::GetInfo()
+{
+	PString result;
+	
+	if (m_sqlConn == NULL)
+		result += "  No SQL connection available\r\n";
+	else {
+		GkSQLConnection::Info info;
+		m_sqlConn->GetInfo(info);
+		result += "  Connected to an SQL Backend: " + PString(info.m_connected ? "Yes" : "No") + "\r\n";
+		result += "  Min Connection Pool Size:    " + PString(info.m_minPoolSize) + "\r\n";
+		result += "  Max Connection Pool Size:    " + PString(info.m_maxPoolSize) + "\r\n";
+		result += "  Idle Connections:            " + PString(info.m_idleConnections) + "\r\n";
+		result += "  Busy Connections::           " + PString(info.m_busyConnections) + "\r\n";
+		result += "  Waiting Requests:            " + PString(info.m_waitingRequests) + "\r\n";
+	}
+	
+	result += ";\r\n";
+	
+	return result;
+}
 
 SQLAuth::SQLAuth(
 	const char* authName,
@@ -454,36 +503,36 @@ SQLAuth::SQLAuth(
 		PTRACE(0, "SQLAUTH\tFATAL: Shutting down");
 		RasServer::Instance()->Stop();
 		return;
-	} else if (!m_regQuery)
+	} else if (!m_regQuery) {
 		PTRACE(4, "SQLAUTH\t" << GetName() << " RRQ query: " << m_regQuery);
+	}
 
 	m_nbQuery = cfg->GetString(authName, "NbQuery", "");
 	if (m_nbQuery.IsEmpty() && IsRasCheckEnabled(RasInfo<H225_LocationRequest>::flag)) {
 		PTRACE(1, "SQLAUTH\t" << GetName() << " module creation failed: "
-			"no LRQ query configured"
-			);
+			"no LRQ query configured");
 		PTRACE(0, "SQLAUTH\tFATAL: Shutting down");
 		RasServer::Instance()->Stop();
 		return;
-	} else if (!m_nbQuery)
+	} else if (!m_nbQuery) {
 		PTRACE(4, "SQLAUTH\t" << GetName() << " LRQ query: " << m_nbQuery);
+	}
 
 	m_callQuery = cfg->GetString(authName, "CallQuery", "");
 	if (m_callQuery.IsEmpty() && (IsRasCheckEnabled(RasInfo<H225_AdmissionRequest>::flag)
 			|| IsMiscCheckEnabled(e_Setup) || IsMiscCheckEnabled(e_SetupUnreg))) {
 		PTRACE(1, "SQLAUTH\t" << GetName() << " module creation failed: "
-			"no ARQ/Setup query configured"
-			);
+			"no ARQ/Setup query configured");
 		PTRACE(0, "SQLAUTH\tFATAL: Shutting down");
 		RasServer::Instance()->Stop();
 		return;
-	} else if (!m_callQuery)
+	} else if (!m_callQuery) {
 		PTRACE(4, "SQLAUTH\t" << GetName() << " ARQ/Setup query: " << m_callQuery);
+	}
 
 	if (!m_sqlConn->Initialize(cfg, authName)) {
 		PTRACE(0, "SQLAUTH\t" << GetName() << " module creation failed: "
-			"could not connect to the database"
-			);
+			"could not connect to the database");
 		return;
 	}
 }
@@ -679,11 +728,20 @@ int SQLAuth::Check(
 		}
 	}
 
+	PStringArray numbersToDial;
 	iter = FindField(result, "redirectnumber");
 	if (iter != result.end()) {
 		const PString &s = iter->first;
 		if (!s) {
-			authData.SetRouteToAlias(s);
+			numbersToDial = s.Tokenise("; \t", FALSE);
+			if (numbersToDial.GetSize() > 0) {
+				PString rewrittenNumber(numbersToDial[0]);
+				PINDEX pos = rewrittenNumber.Find('=');
+				if (pos != P_MAX_INDEX)
+					rewrittenNumber = rewrittenNumber.Left(pos);
+				authData.SetRouteToAlias(rewrittenNumber);
+				PTRACE(5, traceStr << " - call redirected to the number " << rewrittenNumber);
+			}
 			PTRACE(5, traceStr << " - call redirected to the number " << s);
 		}
 	}
@@ -699,10 +757,18 @@ int SQLAuth::Check(
 					
 				if (GetTransportAddress(tokens[i], GK_DEF_ENDPOINT_SIGNAL_PORT, raddr, port)
 						&& raddr.IsValid() && port != 0) {
-					Route route("SQL", raddr, port);
+					Route route("SQLAuth", raddr, port);
 					route.m_destEndpoint = RegistrationTable::Instance()->FindBySignalAdr(
 						SocketToH225TransportAddr(raddr, port)
 						);
+					if (numbersToDial.GetSize() > 0) {
+						route.m_destNumber = (i < numbersToDial.GetSize() ? numbersToDial[i] : numbersToDial[numbersToDial.GetSize() - 1]);
+						PINDEX pos = route.m_destNumber.Find('=');
+						if (pos != P_MAX_INDEX) {
+							route.m_destOutNumber = route.m_destNumber.Mid(pos + 1);
+							route.m_destNumber = route.m_destNumber.Left(pos);
+						}
+					}
 					authData.m_destinationRoutes.push_back(route);
 					PTRACE(5, traceStr << " - call redirected to the address " <<
 						route.AsString()
@@ -840,6 +906,18 @@ int SQLAuth::Check(
 
 	if (!Toolkit::AsBool(result[0].first)) {
 		authData.m_rejectCause = Q931::CallRejected;
+		GkSQLResult::ResultRow::const_iterator iter = FindField(result, "q931cause");
+		if (iter != result.end()) {
+			const PString &s = iter->first;
+			if (s.GetLength() > 0
+				&& strspn((const char*)s, "0123456789") == (size_t)s.GetLength()) {
+				int cause = s.AsInteger();
+				if (cause > 0 && cause < 128) {
+					authData.m_rejectCause = cause;
+					PTRACE(5, traceStr << " - Q.931 cause set to " << authData.m_rejectCause);
+				}
+			}
+		}
 		return e_fail;
 	}
 
@@ -863,12 +941,20 @@ int SQLAuth::Check(
 		}
 	}
 
+	PStringArray numbersToDial;
 	iter = FindField(result, "redirectnumber");
 	if (iter != result.end()) {
 		const PString &s = iter->first;
 		if (!s) {
-			authData.SetRouteToAlias(s);
-			PTRACE(5, traceStr << " - call redirected to the number " << s);
+			numbersToDial = s.Tokenise("; \t", FALSE);
+			if (numbersToDial.GetSize() > 0) {
+				PString rewrittenNumber(numbersToDial[0]);
+				PINDEX pos = rewrittenNumber.Find('=');
+				if (pos != P_MAX_INDEX)
+					rewrittenNumber = rewrittenNumber.Left(pos);
+				authData.SetRouteToAlias(rewrittenNumber);
+				PTRACE(5, traceStr << " - call redirected to the number " << rewrittenNumber);
+			}
 		}
 	}
 			
@@ -883,10 +969,18 @@ int SQLAuth::Check(
 					
 				if (GetTransportAddress(tokens[i], GK_DEF_ENDPOINT_SIGNAL_PORT, raddr, port)
 						&& raddr.IsValid() && port != 0) {
-					Route route("SQL", raddr, port);
+					Route route("SQLAuth", raddr, port);
 					route.m_destEndpoint = RegistrationTable::Instance()->FindBySignalAdr(
 						SocketToH225TransportAddr(raddr, port)
 						);
+					if (numbersToDial.GetSize() > 0) {
+						route.m_destNumber = (i < numbersToDial.GetSize() ? numbersToDial[i] : numbersToDial[numbersToDial.GetSize() - 1]);
+						PINDEX pos = route.m_destNumber.Find('=');
+						if (pos != P_MAX_INDEX) {
+							route.m_destOutNumber = route.m_destNumber.Mid(pos + 1);
+							route.m_destNumber = route.m_destNumber.Left(pos);
+						}
+					}
 					authData.m_destinationRoutes.push_back(route);
 					PTRACE(5, traceStr << " - call redirected to the address " <<
 						route.AsString()
@@ -911,6 +1005,27 @@ int SQLAuth::Check(
 	return e_ok;
 }
 
+PString SQLAuth::GetInfo()
+{
+	PString result;
+	
+	if (m_sqlConn == NULL)
+		result += "  No SQL connection available\r\n";
+	else {
+		GkSQLConnection::Info info;
+		m_sqlConn->GetInfo(info);
+		result += "  Connected to an SQL Backend: " + PString(info.m_connected ? "Yes" : "No") + "\r\n";
+		result += "  Min Connection Pool Size:    " + PString(info.m_minPoolSize) + "\r\n";
+		result += "  Max Connection Pool Size:    " + PString(info.m_maxPoolSize) + "\r\n";
+		result += "  Idle Connections:            " + PString(info.m_idleConnections) + "\r\n";
+		result += "  Busy Connections::           " + PString(info.m_busyConnections) + "\r\n";
+		result += "  Waiting Requests:            " + PString(info.m_waitingRequests) + "\r\n";
+	}
+	
+	result += ";\r\n";
+	
+	return result;
+}
 
 namespace { // anonymous namespace
 	GkAuthCreator<SQLPasswordAuth> SQLPasswordAuthCreator("SQLPasswordAuth");
