@@ -12,7 +12,7 @@
 //////////////////////////////////////////////////////////////////
 
 #ifndef STL_SUPP_H
-#define STL_SUPP_H "@(#) $Id: stl_supp.h,v 1.20 2007/03/21 22:30:16 willamowius Exp $"
+#define STL_SUPP_H "@(#) $Id: stl_supp.h,v 1.21 2008/09/17 20:30:49 willamowius Exp $"
 
 #include <string>
 #include <iterator>
@@ -21,12 +21,6 @@
 
 namespace std {
 
-// Oops... the standard STL minus object is too restricted
-template <class _Tp, class R>
-struct Minus : public binary_function<_Tp,_Tp, R> {
-  R operator()(const _Tp& __x, const _Tp& __y) const { return __x - __y; }
-};
-
 #ifdef _WIN32
 #if (_MSC_VER <= 1200)
 // VC++ didn't define these
