@@ -15,13 +15,13 @@
 //////////////////////////////////////////////////////////////////
 
 #ifndef YASOCKET_H
-#define YASOCKET_H "@(#) $Id: yasocket.h,v 1.29 2008/03/13 13:42:22 zvision Exp $"
+#define YASOCKET_H "@(#) $Id: yasocket.h,v 1.31 2008/04/25 02:51:56 shorne Exp $"
 
 #include <list>
 #include <vector>
 #include <ptlib/sockets.h>
 #include "job.h"
-#include "pwlib_compat.h"
+#include "config.h"
 
 #ifdef LARGE_FDSET
 
@@ -49,8 +49,8 @@ public:
 
 	void SetPort(WORD pt) { port = pt; }
 	WORD GetPort() const { return port; }
-	void GetLocalAddress(Address &) const;
-	void GetLocalAddress(Address &, WORD &) const;
+	virtual void GetLocalAddress(Address &) const;
+	virtual void GetLocalAddress(Address &, WORD &) const;
 	bool SetOption(int, int, int = SOL_SOCKET);
 	bool SetOption(int, const void *, int, int = SOL_SOCKET);
 	bool GetOption(int, int &, int = SOL_SOCKET);
